@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.52
- * 1994/12/03 01:51:39
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopher/CURcurses.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.61 $
+ * $Date: 1996/01/04 18:28:51 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopher/CURcurses.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,35 @@
  * Abstraction of all Curses Functions
  *********************************************************************
  * Revision History:
- * CURcurses.c,v
+ * $Log: CURcurses.c,v $
+ * Revision 3.61  1996/01/04  18:28:51  lindner
+ * Updates for autoconf
+ *
+ * Revision 3.60  1995/11/03  21:18:15  lindner
+ * ANSIfication
+ *
+ * Revision 3.59  1995/11/03  20:50:47  lindner
+ * Coen: fixes..
+ *
+ * Revision 3.58  1995/05/02  06:06:57  lindner
+ * Fix for message str..
+ *
+ * Revision 3.57  1995/04/15  07:09:46  lindner
+ * New funky C-x stuff
+ *
+ * Revision 3.56  1995/03/09  05:24:39  lindner
+ * F.Macrides 01-Mar-1995 Removed the CURsetScreen(cur, initscr()) that
+ * someone #ifdef'ed for VMS in CURresize().
+ *
+ * Revision 3.55  1995/02/27  17:44:51  lindner
+ * New pacbell fcn CURrightline
+ *
+ * Revision 3.54  1995/02/07  19:34:41  lindner
+ * Fix for unix systems without SIGTSTP
+ *
+ * Revision 3.53  1995/01/25  23:09:24  lindner
+ * Word wrapping in gopher forms, provided by Pacific Bell Knowledge Net
+ *
  * Revision 3.52  1994/12/03  01:51:39  lindner
  * Fix prototype for DCLspawn_exception
  *
@@ -260,6 +288,8 @@
 #include "Stdlib.h"
 #include "compatible.h"
 #include "Debug.h"
+#include "String.h"
+#include "util.h"
 
 #ifdef VMS
 static int w_getch();
@@ -274,14 +304,40 @@ void VMSexit();
 int spawn_DCLprocess();
 unsigned int DCLspawn_exception();
 #endif /* VMS */
- 
+
+
+#ifdef CTRLCPROMPTS
+#  define DIALOGCANCELSTR "Cancel - ^C"
+#  define HELPCANCEL      "^C      : Cancel"
+#  define HELP2CANCEL     "^C        : Cancel"
+#  define FORMCANCEL      "[Help: ^-]  [Cancel: ^C] "
+#else 
+#  define DIALOGCANCELSTR "Cancel - ^G"
+#  define HELPCANCEL      "^G      : Cancel"
+#  define HELP2CANCEL     "^G        : Cancel"
+#  define FORMCANCEL      "[Help: ^-]  [Cancel: ^G] "
+#endif /*NEWCTRLC*/
+
+
+#ifdef CONTROLX
+#  define CXPROMPT1    "^X      : Accept"
+#else
+#  define CXPROMPT1    "Enter   : Accept"
+#endif
+
+/*
+ * Do we use wordwrap?
+ */
+
+boolean wordwrap = 0;
+
 
 /*
  * Initialize data space for various screen information
  */
 
 CursesObj *
-CURnew()
+CURnew(void)
 {
      CursesObj *cur;
 
@@ -311,8 +367,7 @@ CURnew()
  */
 
 void
-CURinit(cur)
-  CursesObj *cur;
+CURinit(CursesObj *cur)
 {
 #ifdef VMS
      VMSCURinit(cur);
@@ -385,9 +440,7 @@ static char *CURgraphicChars[] = {
 };
 
 void
-CURsetGraphicsChars(cur, termtype)
-  CursesObj *cur;
-  char *termtype;
+CURsetGraphicsChars(CursesObj *cur, char *termtype)
 {
      cur->Box_hline  = '-';
      cur->Box_vline  = '|';
@@ -420,6 +473,19 @@ CURsetGraphicsChars(cur, termtype)
 	  while ((term = CURgraphicChars[count++]) != NULL) {
 	       chars = CURgraphicChars[count++];
 	       if (strcmp(termname, term) == 0) {
+#ifdef ACS_BSSB
+		    cur->Box_hline  = ACS_HLINE;
+		    cur->Box_vline  = ACS_VLINE;
+		    cur->Box_ul     = ACS_BSSB;
+		    cur->Box_ur     = ACS_BBSS;
+		    cur->Box_ll     = ACS_SSBB;
+		    cur->Box_lr     = ACS_SBBS;
+		    cur->Box_tt     = ACS_TTEE;
+		    cur->Box_bt     = ACS_BTEE;
+		    cur->Box_lt     = ACS_LTEE;
+		    cur->Box_rt     = ACS_RTEE;
+		    cur->Box_ct     = ACS_PLUS;
+#else /* ACS_BSSB */
 		    cur->Box_hline  = A_ALTCHARSET|chars[0];
 		    cur->Box_vline  = A_ALTCHARSET|chars[1];
 		    cur->Box_ul     = A_ALTCHARSET|chars[2];
@@ -431,6 +497,8 @@ CURsetGraphicsChars(cur, termtype)
 		    cur->Box_lt     = A_ALTCHARSET|chars[8];
 		    cur->Box_rt     = A_ALTCHARSET|chars[9];
 		    cur->Box_ct     = A_ALTCHARSET|chars[10];
+#endif /* ACS_BSSB */
+
 	       }
 	  }
      }
@@ -444,8 +512,7 @@ CURsetGraphicsChars(cur, termtype)
  */
 
 void
-CURenter(cur)
-  CursesObj *cur;
+CURenter(CursesObj *cur)
 {
      /* for safety */
      if (cur->inCurses == TRUE)
@@ -467,7 +534,7 @@ CURenter(cur)
      if (cur->sigwinch != SIG_ERR)
 	  signal(SIGWINCH, cur->sigwinch);
 #endif
-#ifndef VMS
+#ifdef SIGTSTP
      if (cur->sigtstp != SIG_ERR)
 	  signal(SIGTSTP, cur->sigtstp);
 #endif
@@ -478,9 +545,7 @@ CURenter(cur)
  */
 
 void
-CURwenter(cur, win)
-  CursesObj *cur;
-  WINDOW *win;
+CURwenter(CursesObj *cur, WINDOW *win)
 {
      cbreak();
      noecho();
@@ -500,8 +565,7 @@ CURwenter(cur, win)
  */
 
 void
-CURexit(cur)
-  CursesObj *cur;
+CURexit(CursesObj *cur)
 {
      
      if (!cur->inCurses)
@@ -519,7 +583,9 @@ CURexit(cur)
 
 #ifdef VMS
      (void)resetterm();
-#else
+#endif
+
+#ifdef SIGTSTP
      tputs(CURgetCLS(cur),1,CURoutchar);
      fflush(stdout);
  
@@ -539,8 +605,7 @@ CURexit(cur)
  */
 
 int
-CURoutchar(c)
-  char c;
+CURoutchar(char c)
 {
      /** output the given character.  From tputs... **/
      /** Note: this CANNOT be a macro!              **/
@@ -549,16 +614,21 @@ CURoutchar(c)
      return(c);
 }
 
+/*
+ * Rightline, uses curses routines to center a line.
+ */
+void CURrightline(CursesObj *cur, char *theline, int yval)
+{
+     mvaddstr(yval, (COLS - strlen(theline) -1), theline);
+}
+
+
+
 
 /*
  * Centerline, uses curses routines to center a line.
  */
-void CURcenterline(cur, win, theline, yval, windowidth, bright)
-  CursesObj *cur;
-  WINDOW    *win;
-  char      *theline;
-  int       yval, windowidth;
-  boolean   bright;
+void CURcenterline(CursesObj *cur, WINDOW *win, char *theline, int yval, int windowidth, boolean bright)
 {
      char TrimmedLine[256];
 
@@ -592,12 +662,7 @@ void CURcenterline(cur, win, theline, yval, windowidth, bright)
 
 
 int
-CURwgetstr(cur, win, inputline, maxlength, hidden)
-  CursesObj *cur;
-  WINDOW    *win;
-  char      *inputline;
-  int       maxlength;
-  boolean   hidden;
+CURwgetstr(CursesObj *cur, WINDOW *win, char *inputline, int maxlength, boolean hidden)
 {
      int pointer = 0;
      int curpointer = 0;
@@ -624,6 +689,9 @@ CURwgetstr(cur, win, inputline, maxlength, hidden)
 	  switch (ch) {
 
 	  case '\n':
+#ifdef CONTROLX
+	  case '\030':
+#endif
 	       inputline[pointer] = '\0';
 	       wstandend(win);
 	       return(ch);
@@ -722,9 +790,18 @@ CURwgetstr(cur, win, inputline, maxlength, hidden)
 	       break;
 
 	  default:
-	       if (ch < 256 && isprint(ch) && curpointer == maxlength) {
-		    CURBeep(cur);
+	       if (ch < 256 && isprint(ch) && curpointer >= maxlength) {
+		    /* Return at the end for word wrapping */
+		    if (wordwrap) {
+			 /*inputline[curpointer] = ch;*/
+			 inputline[curpointer] = '\0';
+			 wstandend(win);
+			 wrefresh(win);
+			 return(ch);
+		    } else
+			 CURBeep(cur);
 	       }
+
 	       else if (ch < 256 && isprint(ch)) {
 
 		    inputline[curpointer++] = ch;
@@ -753,15 +830,29 @@ CURwgetstr(cur, win, inputline, maxlength, hidden)
  */
 
 int
-CURwgetch(cur, window)
-  CursesObj *cur;
-  WINDOW *window;
+CURwgetch(CursesObj *cur, WINDOW *window)
 {
      int a, b, c;
+
+     if (ControlCpressed) {
+	  return ('\007');  /** Control-C cancels us **/
+     }
+     
+     if (signal(SIGINT, controlcJmp) == SIG_ERR)
+	  perror("signal died:\n"), exit(-1);
+     
+     if (setjmp(Jmpenv)) {
+	  /* Note controlcJmp will reprime controlc and set flag*/
+	  Debug("interruptable_wgetch triggered\r\n",NULL);
+	  ControlCpressed = FALSE;	/* Deal with return code, not flag*/
+	  return('\007');
+     }
+
      
      while (1) {
 
 	  c = wgetch(window);
+	  reprimeControlc();
 
 	  switch (c) {
 
@@ -831,6 +922,7 @@ CURwgetch(cur, window)
 	       /* fall through to default to reassign certain keys */
 
 	  default:
+	       
 	  /* The many forms of the return key... */
 	  if ((c == KEY_ENTER)|| (c=='\r')) 
 	       c = '\n'; /** SYSV curses Gack! **/
@@ -847,8 +939,7 @@ CURwgetch(cur, window)
 }
 
 int
-CURgetch(cur)
-  CursesObj *cur;
+CURgetch(CursesObj *cur)
 {
    return(CURwgetch(cur, stdscr));
 }  
@@ -858,14 +949,10 @@ CURgetch(cur)
  */
 
 void
-CURresize(cur)
-  CursesObj *cur;
+CURresize(CursesObj *cur)
 {
      if (cur->inCurses) {
 	  CURexit(cur);
-#ifdef VMS
-	  CURsetScreen(cur, initscr());
-#endif
 	  CURenter(cur);
      }
 }
@@ -880,9 +967,7 @@ CURresize(cur)
  */
 
 int
-CURGetOneOption(cur, Title, OptionName, Response)
-  CursesObj *cur;
-  char *Title, *OptionName, *Response;
+CURGetOneOption(CursesObj *cur, char *Title, char *OptionName, char *Response)
 {
      int i;
      Requestitem *items[2], item;
@@ -905,9 +990,7 @@ CURGetOneOption(cur, Title, OptionName, Response)
  * garsh darn terminal is just too gadblam slow :-)
  */
 int
-CUROldGetOneOption(cur, OptionName, Response)
-  CursesObj *cur;
-  char *OptionName, *Response;
+CUROldGetOneOption(CursesObj *cur, char *OptionName, char *Response)
 {
      int i;
      
@@ -931,9 +1014,7 @@ CUROldGetOneOption(cur, OptionName, Response)
  */
 
 void
-CURgetYesorNo(cur, OptionName, Response)
-  CursesObj *cur;
-  char *OptionName, *Response;
+CURgetYesorNo(CursesObj *cur, char *OptionName, char *Response)
 {
      int c;
      int posx, posy;
@@ -989,8 +1070,7 @@ CURgetYesorNo(cur, OptionName, Response)
 }
 	  
 void 
-CURBeep(cur)
-  CursesObj *cur;
+CURBeep(CursesObj *cur)
 {
 	Debugmsg("CURBeep\r\n")
 #ifdef SYSVCURSES
@@ -1004,10 +1084,7 @@ CURBeep(cur)
 
 
 void
-CURbox(cur, win, height, width)
-  CursesObj *cur;
-  WINDOW *win;
-  int width, height;
+CURbox(CursesObj *cur, WINDOW *win, int height, int width)
 {
      int i;
 
@@ -1033,11 +1110,7 @@ CURbox(cur, win, height, width)
 
 
 void
-CURbutton(cur, win, Label, bright)
-  CursesObj *cur;
-  WINDOW *win;
-  char *Label;
-  boolean bright;
+CURbutton(CursesObj *cur, WINDOW *win, char *Label, boolean bright)
 {
 /*     wboldout(win);*/
 
@@ -1057,10 +1130,7 @@ CURbutton(cur, win, Label, bright)
 
 
 int
-CURDialog(cur, Wintitle, Message)
-  CursesObj *cur;
-  char **Message;
-  char *Wintitle;
+CURDialog(CursesObj *cur, char *Wintitle, char **Message)
 {
      int i;
      int messheight=0;
@@ -1080,6 +1150,10 @@ CURDialog(cur, Wintitle, Message)
 
      tempwin = newwin(5+messheight, winwidth, (LINES-(5+messheight))/2,
      			(COLS-winwidth)/2);
+
+     if (tempwin == (WINDOW *)0)
+	  return(-1);
+
      CURwenter(cur,tempwin);
      CURbox(cur, tempwin, 5+messheight, winwidth);
 
@@ -1106,7 +1180,7 @@ CURDialog(cur, Wintitle, Message)
 
      /** Add the keyboard labels **/
      wmove(tempwin, 3+messheight, winwidth - 28);
-     CURbutton(cur, tempwin, Gtxt("Cancel: ^G",211), FALSE);
+     CURbutton(cur, tempwin, Gtxt(DIALOGCANCELSTR,211), FALSE);
      waddch(tempwin, ' ');
      CURbutton(cur, tempwin, Gtxt("OK: Enter",191), FALSE);
 
@@ -1125,11 +1199,7 @@ CURDialog(cur, Wintitle, Message)
 
 
 int
-CURRequest(cur,Wintitle,Prompts,Stowages)
-  CursesObj *cur;
-  char *Wintitle;
-  char **Prompts;
-  char **Stowages;
+CURRequest(CursesObj *cur, char *Wintitle, char **Prompts, char **Stowages)
 {
      Requestitem **items;
      int i, choice, numprompts = 0;
@@ -1168,10 +1238,7 @@ CURRequest(cur,Wintitle,Prompts,Stowages)
  */
 
 int
-CURrequester(cur, Wintitle, items)
-  CursesObj   *cur;
-  char        *Wintitle;
-  Requestitem **items;
+CURrequester(CursesObj *cur, char *Wintitle, Requestitem **items)
 {
      int         BottomField;
      int         ch;
@@ -1192,13 +1259,13 @@ CURrequester(cur, Wintitle, items)
 
      if (RequestHelp == NULL) {
 	  RequestHelp = (char**) malloc(sizeof(char*) * 8);
-	  RequestHelp[0] = Gtxt("^G      : Cancel",190);
+	  RequestHelp[0] = Gtxt(HELPCANCEL,190);
 	  RequestHelp[1] = Gtxt("Tab, ^N : Move to next field",192);
 	  RequestHelp[2] = Gtxt("^P      : Move to previous field",193);
 	  RequestHelp[3] = Gtxt("^F      : Display next page",194);
 	  RequestHelp[4] = Gtxt("^B      : Display previous page",195);
 	  RequestHelp[5] = Gtxt("^-      : Help (^/ or ^7 may work)",196);
-	  RequestHelp[6] = Gtxt("Enter   : Accept",197);
+	  RequestHelp[6] = Gtxt(CXPROMPT1,197);
 	  RequestHelp[7] = NULL;
      }
 
@@ -1207,7 +1274,7 @@ CURrequester(cur, Wintitle, items)
 
 	  /*** Skip non editable prompts ***/
 	  if (curitem->thing != CUR_LABEL) {
-	       if (strlen(curitem->prompt) > maxpromptwidth)
+	       if (curitem->prompt && strlen(curitem->prompt) > maxpromptwidth)
 		    maxpromptwidth = strlen(curitem->prompt);
 	  } else {
 	       if (currentfield == numprompts-1)
@@ -1246,11 +1313,19 @@ CURrequester(cur, Wintitle, items)
      while (1) {
           /*** Add the labels ***/
           wmove(tempwin, WinLines - 2, 2);
-          waddstr(tempwin, Gtxt("[Help: ^-]  [Cancel: ^G] ",198));
+          waddstr(tempwin, Gtxt(FORMCANCEL,198));
           if (items[currentfield]->thing == CUR_CHOICE)
                waddstr(tempwin, Gtxt("  [Cycle Values: Space]  [List Values: l]",199));
-          else
-               waddstr(tempwin, "                                         ");
+          else {
+#ifdef CONTROLX
+	       waddstr(tempwin, " [Accept: ^X] ");
+#else
+	       waddstr(tempwin, " [Accept: Enter] ");
+#endif
+	       waddstr(tempwin, " [Next field: TAB] ");
+	       waddstr(tempwin, "           ");
+	  }
+
 
           /** Add the prompts and typing area **/
           if (numprompts == 1 && items[0]->thing == CUR_PROMPT) {
@@ -1274,10 +1349,13 @@ CURrequester(cur, Wintitle, items)
                i = 0;
                for (k = TopField; k <= BottomField ; k++) {
 	            wmove(tempwin, 2 + i, 2);
-	            waddstr(tempwin, items[k]->prompt);
+		    if (items[k]->prompt) {
+			 
+			 waddstr(tempwin, items[k]->prompt);
 	  
-		    for (j = WinWidth - strlen(items[k]->prompt) - 4; j > 0; j--)
-			 waddch(tempwin, ' ');
+			 for (j = WinWidth-strlen(items[k]->prompt)-4; j>0;j--)
+			      waddch(tempwin, ' ');
+		    }
 
 	            switch (items[k]->thing) {
 
@@ -1292,6 +1370,7 @@ CURrequester(cur, Wintitle, items)
 		         break;
 			 
 		    case CUR_ASKL:
+			 wordwrap = TRUE;
 			 wmove(tempwin, 2 + i, 2);
 	                 wstandout(tempwin);
 			 waddstr(tempwin, items[k]->stowage);
@@ -1305,17 +1384,21 @@ CURrequester(cur, Wintitle, items)
 	            default:
 	                 /** Add the black space for the stowage, **/
 	                 /** and the stowage, if it exists        **/
+			 wordwrap = FALSE;
 	                 wmove(tempwin, 2 + i, maxpromptwidth + 4);
 	                 wstandout(tempwin);
 
 	                 if (items[k]->thing == CUR_PASSWD)
 			      for (j = strlen(items[k]->stowage); j > 0; j--)
 			           waddch(tempwin, '*');
-	                 else
+	                 else if (items[k]->stowage)
 		              waddstr(tempwin, items[k]->stowage);
 
-			 for (j = WinWidth - strlen(items[k]->stowage) - 
-			      maxpromptwidth - 6; j > 0; j--)
+
+
+			 for (j = WinWidth - 
+			      (items[k]->stowage ? strlen(items[k]->stowage):0)
+			      - maxpromptwidth - 6; j > 0; j--)
 		              waddch(tempwin, ' ');
 
 	                 wstandend(tempwin);
@@ -1386,17 +1469,26 @@ CURrequester(cur, Wintitle, items)
 	       int zemax = maxlength;
 
 	       if (items[currentfield]->thing == CUR_ASKL)
-		    zemax = COLS - 4;
+		    zemax = COLS - 6;
 		    
 	       ch = CURwgetstr(cur,tempwin,items[currentfield]->stowage,
                     zemax, items[currentfield]->thing == CUR_PASSWD);
 	  }
 
-	  switch (ch) {
+#ifdef CONTROLX
+	  if (ch == '\n') {
+	       if (numprompts > 1) {
+		    ch = '\t';	   /* Treat as tab, next field */
+	       } else {
+		    ch = '\030';      /* Treat as control-X, accept */
+	       }
+	  }
+#endif
 
+
+	  switch (ch) {
           case '\016':
 	  case '\t':
-/*	  case '\n':*/
 	       /*** Move to next non-label field (tab) ***/
 	       do {
                     currentfield++;
@@ -1570,13 +1662,58 @@ CURrequester(cur, Wintitle, items)
 	       delwin(tempwin);
 	       return(-1);
 	       
+#ifdef CONTROLX
+	  case '\030':
+#else
 	  case '\n':
+#endif
 	       delwin(tempwin);
 	       return(0);
 
           default:
-               CURBeep(cur);
-               break;
+	       if (wordwrap) {
+		    char *p;
+		    if ( currentfield == numprompts-1){
+			 CURBeep(cur);
+			 continue;
+		    }
+		    if (items[currentfield+1]->thing != CUR_ASKL ) {
+			 CURBeep(cur);
+			 continue;
+		    }
+
+		    /* we must go and find the last space in stowage and move
+		       to next line */
+		    p=strrchr(items[currentfield]->stowage,' ');
+
+		    /* copy rest of string to next line */
+		    if ( p != NULL ) {
+			 char addit[2];
+			 *addit = ch;
+			 *(addit+1) = '\0';
+			 strcat(p+1, addit);
+
+			 strcpy(items[currentfield+1]->stowage,p+1);
+			 
+			 /* now blank out those characters */
+			 *p='\0';
+			 wmove(tempwin, 2 + currentfield - TopField, 2); 
+			 wstandout(tempwin);
+			 waddstr(tempwin, items[currentfield]->stowage);
+			 for (j=strlen(items[currentfield]->stowage)+
+			      maxpromptwidth+4; j < COLS-6; j++)
+			      waddch(tempwin, ' ');
+			 wstandend(tempwin);
+		    }
+		    currentfield++;	/* go to next ASKL field */
+
+                    if (currentfield > BottomField) {
+                         TopField++;
+                         BottomField++;
+
+                    }
+
+	       }
 	  }
      }
 }
@@ -1593,12 +1730,7 @@ CURrequester(cur, Wintitle, items)
 
 
 int
-CURChoice(cur, WinTitle, Choices, Prompt, DefaultChoice)
-  CursesObj *cur;
-  char      *WinTitle;
-  char      **Choices;
-  char      *Prompt;
-  int       DefaultChoice;
+CURChoice(CursesObj *cur, char *WinTitle, char **Choices, char *Prompt, int DefaultChoice)
 {
      int    BottomChoice;
      int    ch;
@@ -1621,7 +1753,7 @@ CURChoice(cur, WinTitle, Choices, Prompt, DefaultChoice)
 
      if (ChoiceHelp == NULL) {
 	  ChoiceHelp = (char**) malloc(sizeof(char*) * 10);
-	  ChoiceHelp[0] = Gtxt("^G        : Cancel",202);
+	  ChoiceHelp[0] = Gtxt(HELP2CANCEL,202);
 	  ChoiceHelp[1] = Gtxt("Down, ^N  : Move to next choice",203);
 	  ChoiceHelp[2] = Gtxt("Up, ^P    : Move to previous choice",204);
 	  ChoiceHelp[3] = Gtxt("Space, ^F : Display next page",205);
@@ -1685,7 +1817,7 @@ CURChoice(cur, WinTitle, Choices, Prompt, DefaultChoice)
      wmove(TempWin, ChoicesPerPage + 4, 3);
      CURbutton(cur, TempWin, Gtxt("Help: ?",96), FALSE);
      waddstr(TempWin, "  ");
-     CURbutton(cur, TempWin, Gtxt("Cancel: ^G",211), FALSE);
+     CURbutton(cur, TempWin, Gtxt(DIALOGCANCELSTR,211), FALSE);
 
      /*** Display the dialog box and allow the user to make a choice ***/
      while (1) {
@@ -1838,6 +1970,7 @@ CURChoice(cur, WinTitle, Choices, Prompt, DefaultChoice)
                return (-1);
 
           case '\n':
+	  case '\030':  /* Control X */
                /*** Accept ***/
                delwin(TempWin);
                return (CurrentChoice);
