@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.105
- * 1994/12/07 07:43:22
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopher/gopher.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.112 $
+ * $Date: 1996/01/04 18:28:53 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopher/gopher.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,30 @@
  * Main functions for the gopher client
  *********************************************************************
  * Revision History:
- * gopher.c,v
+ * $Log: gopher.c,v $
+ * Revision 3.112  1996/01/04  18:28:53  lindner
+ * Updates for autoconf
+ *
+ * Revision 3.111  1995/11/03  21:18:18  lindner
+ * ANSIfication
+ *
+ * Revision 3.110  1995/11/03  20:50:48  lindner
+ * Coen: fixes..
+ *
+ * Revision 3.109  1995/06/08  05:21:47  lindner
+ * Fixed conflict for message 221, fixed problem with DECC
+ *
+ * Revision 3.108  1995/04/15  07:09:02  lindner
+ * New control C methods, CTRL-X mods
+ *
+ * Revision 3.107  1995/02/27  17:09:49  lindner
+ * Add Pacbell NOBANNER patch
+ *
+ * Revision 3.106  1995/01/20  04:52:24  lindner
+ * Modifications for SOCKS
+ * Use VARIABLE length records for text on VMS
+ * Don't add the default gopher server when using the -b option.
+ *
  * Revision 3.105  1994/12/07  07:43:22  lindner
  * Fix for URLs on the command line from F.Macrdes
  *
@@ -425,8 +448,8 @@ static	CursesObj	*LangScreen = NULL;
 #include "Malloc.h"
 #include "patchlevel.h"
 
-void describe_gopher();
-extern int  twirl();
+void describe_gopher(char *banner, GopherStruct *ZeGopher, FILE *gripefile);
+extern int  twirl(void);
 
 
 /*
@@ -434,8 +457,7 @@ extern int  twirl();
 */
 
 void
-do_tel_3270(ZeGopher)
-  GopherStruct *ZeGopher;
+do_tel_3270(GopherStruct *ZeGopher)
 {
      char *Dialogmess[9];
 
@@ -560,7 +582,7 @@ trace_telnet(msg, gs)
 	  inited = TRUE;
      }
      if (gs != NULL) {
-	  url = GSgetURL(gs);
+	  url = GSgetURL(gs, "");
 	  name = GSgetTitle(gs);
      }
 
@@ -633,18 +655,18 @@ static
 
 
 void
-Gripe(gs)
- GopherObj *gs;
+Gripe(GopherObj *gs)
 {
      char *gripeprompt[15];
      char *gripemess[15];
      char *cp = NULL;
-     char  email[128], mailcmd[256], version[128];
-     int   i;
+     char  email[128], version[128];
+     int   i, j;
      char *mailargv[3];
      FileIO *fio;
 
 #ifdef VMS
+     char mailcmd[256],
      FILE *f;
      char MailAddress[512], *tmpfilename;
 #endif
@@ -706,8 +728,15 @@ Gripe(gs)
      for (i=0; i < 3; i++)
 	  gripemess[i] = NULL;
 
+     
+
      gripeprompt[0] = Gtxt("Hit the Tab key at the end of each line you type.",49);
+#ifdef CONTROLX
+     gripeprompt[1] = Gtxt("Press Control-X to send your message.",50);
+#else
      gripeprompt[1] = Gtxt("Hit the Enter key to send your message.",50);
+#endif
+
 
 #ifdef MODIFIABLE_GRIPE_TO
      gripeprompt[2] = Gtxt("To",230);
@@ -723,6 +752,16 @@ Gripe(gs)
      if (CURRequest(CursesScreen, GSgetAdmin(gs), gripeprompt, gripemess)!=0) {
 	  return;
      }
+     if (strlen(gripemess[3]) == 0) {
+	  CursesErrorMsg(Gtxt("Cannot send, no subject...",228));
+	  return;
+     }
+     for (i=4, j=0; i<14; i++)
+	  j += strlen(gripemess[i]);
+     if (j < 20) {
+	  CursesErrorMsg(Gtxt("Cannot send, message too small...",228));
+	  return;
+     }
 
 #ifdef MODIFIABLE_GRIPE_TO
      if (strlen(strcpy(email,gripemess[2])) == 0) {
@@ -830,8 +869,7 @@ Gripe(gs)
 ** make a copy if they want to call do_index multiple times.
 */
 
-char* do_index(ZeGopher)
-  GopherObj *ZeGopher;
+char* do_index(GopherObj *ZeGopher)
 {
      static char *inputline = NULL;
      static char *prompt[2];
@@ -869,8 +907,7 @@ char* do_index(ZeGopher)
 
 #ifndef VMS
 void
-suck_sound(sockfd)
-  int sockfd;
+suck_sound(int sockfd)
 {
      FileIO *Play;
      int j;
@@ -907,8 +944,7 @@ suck_sound(sockfd)
  */
 
 void
-do_sound(ZeGopher)
-  GopherStruct *ZeGopher;
+do_sound(GopherStruct *ZeGopher)
 {
 #ifdef VMS
      CursesErrorMsg(Gtxt("Sorry, this machine doesn't support sounds",159));
@@ -953,8 +989,7 @@ do_sound(ZeGopher)
 **/
 
 void
-showfile(ZeGopher)
-  GopherObj *ZeGopher;
+showfile(GopherObj *ZeGopher)
 {
      char    *tmpfilename = NULL;
      FILE    *tmpfile;
@@ -1006,9 +1041,7 @@ showfile(ZeGopher)
 #endif
 
 #if defined(VMS)  && defined(VMSRecords)
-	       if (GSgetType(ZeGopher) == A_FILE ||
-	           GSgetType(ZeGopher) == A_MIME ||
-		   GSgetType(ZeGopher) == A_CSO)
+	       if (GSisText(ZeGopher, view))
 		    /*** Use VARIABLE length records for text on VMS ***/
 		    tmpfile = fopen_VAR(tmpfilename, "w");
 	       else
@@ -1051,10 +1084,10 @@ showfile(ZeGopher)
 	  }  /* Forkoff */
 #endif
 	  
- 	  Debug("showfile: view=%s ",view);
- 	  Debug("command=%s ",inputline);
- 	  Debug("file=%s",tmpfilename);
- 	  Debug("/%s\n",GSgetLocalFile(ZeGopher));
+ 	  Debug("\n\rshowfile: view=%s ",view);
+ 	  Debug("\n\rcommand=%s ",inputline);
+ 	  Debug("\n\rfile=%s",tmpfilename);
+ 	  Debug("\n\r/%s\n",GSgetLocalFile(ZeGopher));
 	  
 	  GS2FileSucceeded = GStoFile(ZeGopher, tmpfile, view, twirl);
 
@@ -1105,8 +1138,7 @@ showfile(ZeGopher)
  */
 
 void
-do_movie(ZeGopher)
-  GopherStruct *ZeGopher;
+do_movie(GopherStruct *ZeGopher)
 {
      showfile(ZeGopher);
 }
@@ -1119,8 +1151,7 @@ do_movie(ZeGopher)
 */
 
 void
-pushgopher(ZeDir)
-  GopherDirObj *ZeDir;
+pushgopher(GopherDirObj *ZeDir)
 {
 
      OldDirs[iLevel]= ZeDir;
@@ -1132,8 +1163,7 @@ pushgopher(ZeDir)
 */
 
 int
-popgopher(ZeDir)
-  GopherDirObj **ZeDir;
+popgopher(GopherDirObj **ZeDir)
 {
 
      if (iLevel == 0)
@@ -1156,10 +1186,7 @@ extern char *sys_errlist[];
 extern int  errno;
 #endif
 
-void check_sock(sockfd, host, port)
-  int sockfd;
-  char *host;
-  int port;
+void check_sock(int sockfd, char *host, int port)
 {
      char DispString[WHOLELINE];
      char DispString2[WHOLELINE];
@@ -1185,7 +1212,11 @@ void check_sock(sockfd, host, port)
 	       sprintf(DispString2, "%.78s.", vms_errno_string());
 	       DispStrings[2] = DispString2;
 #else
+#  ifdef __DECC
+	       if (errno > 0 && errno <= 65) {
+#  else
 	       if (errno > 0 && errno <= sys_nerr) {
+#  endif
 		    sprintf(DispString2, Gtxt("Connection failed: %s.",78),
 #ifdef VMS
 			    strerror(errno));
@@ -1213,7 +1244,7 @@ void check_sock(sockfd, host, port)
 
 
 BOOLEAN
-ReallyQuit()
+ReallyQuit(void)
 {
      char yesno[3];
 	       
@@ -1229,15 +1260,24 @@ ReallyQuit()
 }
      
 
-void
-CleanupandExit(exitval)
-  int exitval;
+RETSIGTYPE
+CleanupandExit(int exitval)
 {
      GopherDirObj *gd;
 #ifdef VMS
      extern boolean DidCleanup;
 #endif
 
+#ifdef SIGWINCH
+     signal(SIGWINCH, SIG_IGN);
+#endif
+     signal(SIGINT, SIG_IGN);
+     signal(SIGQUIT, SIG_IGN);
+     signal(SIGPIPE, SIG_IGN);
+     signal(SIGTERM, SIG_IGN);
+     signal(SIGHUP, SIG_IGN);
+
+
      if (CursesScreen)
 	  CURexit(CursesScreen);
 
@@ -1267,9 +1307,9 @@ CleanupandExit(exitval)
 /**************
 ** This bit of code catches control-c's, it cleans up the curses stuff.
 */
-void
-controlc(sig)
-  int sig;
+
+RETSIGTYPE
+controlc_old(int sig)
 {
 
 #ifdef VMS
@@ -1311,15 +1351,12 @@ controlc(sig)
 }
 
 
-     
-
 /**************
 ** This bit of code catches window size change signals
 **/
 
-void
-sizechange(sig)
-  int sig;
+RETSIGTYPE
+sizechange(int sig)
 {
      int lines, cols;
      
@@ -1366,7 +1403,7 @@ sizechange(sig)
 ***********/
 
 void
-Initialize()
+Initialize(void)
 {
 
      Debugmsg("Initialize\n")
@@ -1448,7 +1485,7 @@ Initialize()
  */
 
 void
-SetOptions()
+SetOptions(void)
 {
      int choice, restricted = FALSE;
      char     *shell;
@@ -1642,9 +1679,7 @@ SetOptions()
 static char *search_string = NULL;
 
 int
-main(argc, argv)
-  int argc;
-  char *argv[];
+main(int argc, char **argv)
 {
      BOOLEAN bDone = FALSE;
      char sTmp[80];
@@ -1699,6 +1734,7 @@ main(argc, argv)
 	  RootGophers[i] = GSnew();
 	  GSsetType (RootGophers[i], A_DIRECTORY);   
 	  GSsetPath (RootGophers[i],"");
+	  GSsetGplus(RootGophers[i], TRUE);
      }
      
      /** Should generalize this to >2 hosts .... Sigh... ***/
@@ -1730,6 +1766,9 @@ main(argc, argv)
 	  case 'p':
 	       GSsetPath(RootGophers[0], optarg);
 	       GSsetPath(RootGophers[1], optarg);
+	       GSsetGplus(RootGophers[0], FALSE);
+	       GSsetGplus(RootGophers[1], FALSE);
+
 	       startAtHome = FALSE;
 	       break;
 	  case 'T':
@@ -1840,6 +1879,10 @@ main(argc, argv)
 
      /*** Set up global variables, etc. ***/
 
+#ifdef SOCKS
+     SOCKSinit(argv[0]);
+#endif
+
      Initialize();
 
      if (startAtHome && (RCgetHome(GlobalRC) != NULL)) {
@@ -1850,8 +1893,6 @@ main(argc, argv)
      if (Bkmarksfirst) {
 	  CurrentDir = RCgetBookmarkDir(GlobalRC);
 	  
-	  if (CurrentDir != NULL)
-	       GDaddGS(CurrentDir, RootGophers[0]);
      } else {
 	  int rnum;
 
@@ -1919,12 +1960,6 @@ main(argc, argv)
 
      while (bDone == FALSE)
      {
-	  /* Set up a longjmp for control-cs **/
-	  if (setjmp(Jmpenv) != 0) {
-	       /* This part only runs when returning from a longjmp */
-	       Garbled = TRUE;
-	  }
-
 	  GetMenu(CurrentDir, &TypedChar, Garbled);
 	  Garbled = TRUE;
 #ifdef DESCRIBE_GOPHER
@@ -2373,6 +2408,11 @@ main(argc, argv)
                   break;
              }
 #endif
+ 	       if (CurrentDir != RCgetBookmarkDir(GlobalRC)) {
+		    Download_file(GDgetEntry(CurrentDir,GDgetCurrentItem(CurrentDir)-1));
+		    break;
+	       }
+
 
 	       logrequest("DELETE BOOKMARK", GDgetEntry(CurrentDir, GDgetCurrentItem(CurrentDir)-1));
 	       
@@ -2408,7 +2448,7 @@ main(argc, argv)
 	            break;
 	       if (setlocale_LangDir== NULL) {
  		    CursesErrorMsg(Gtxt("Sorry, no language choice available",
-					221));
+					239));
 	       }
 	       else
 	       {
@@ -2526,10 +2566,15 @@ main(argc, argv)
 	       /*** Display help file ***/
 	       GopherObj *tmpgs;
 	       char *tmpurl = (char *)malloc(17+255);
+	       char line[80];
 	       
 	       tmpgs = GSnew();
 	       GSsetType(tmpgs, A_FILE);
-	       GSsetTitle(tmpgs, Gtxt("Gopher Help File",92));
+	       sprintf(line, Gtxt("Internet Gopher Information Client v%s.%s.%d",102),
+		       GOPHER_MAJOR_VERSION, GOPHER_MINOR_VERSION, PATCHLEVEL);
+
+/*	       GSsetTitle(tmpgs, Gtxt("Gopher Help File",92));*/
+	       GSsetTitle(tmpgs, line);
 	       
 #ifdef VMS
 	       GSsetLocalFile(tmpgs, Gtxt(GOPHERHELP,223));
@@ -2656,8 +2701,7 @@ setlocale_screen()
 
 
 int
-Load_Index(ZeGopher)
-  GopherObj *ZeGopher;
+Load_Index(GopherObj *ZeGopher)
 {
      Draw_Status(Gtxt("Searching...",143));
      refresh();
@@ -2666,8 +2710,7 @@ Load_Index(ZeGopher)
 }
 
 int
-Load_Dir(ZeGopher)
-  GopherObj *ZeGopher;
+Load_Dir(GopherObj *ZeGopher)
 {
      Searchstring= NULL;
      return(Load_Index_or_Dir(ZeGopher, NULL));
@@ -2675,9 +2718,7 @@ Load_Dir(ZeGopher)
 
 
 int
-Load_Index_or_Dir(ZeGopher, Searchmungestr)
-  GopherObj *ZeGopher;
-  char      *Searchmungestr;
+Load_Index_or_Dir(GopherObj *ZeGopher, char *Searchmungestr)
 {
      int failed = 0;
      int sockfd;
@@ -2691,7 +2732,9 @@ Load_Index_or_Dir(ZeGopher, Searchmungestr)
 
      Draw_Status(Gtxt("Connecting...",77)); refresh();
 
+
      if ((sockfd = GSconnect(ZeGopher)) <0) {
+
 	  check_sock(sockfd, GSgetHost(ZeGopher), GSgetPort(ZeGopher));
 	  failed = 1;
      }
@@ -2755,8 +2798,7 @@ Load_Index_or_Dir(ZeGopher, Searchmungestr)
  */
 
 int
-process_request(ZeGopher)
-  GopherObj *ZeGopher;
+process_request(GopherObj *ZeGopher)
 {
      int failed=0;
      char ImageCmd[128];
@@ -2925,13 +2967,13 @@ process_request(ZeGopher)
 
 void
 #ifdef DESCRIBE_GOPHER_GRIPE
-describe_gopher(banner, ZeGopher, gripefile)
-  FILE *gripefile;
+describe_gopher(char *banner, GopherStruct *ZeGopher, FILE *gripefile)
+                  
 #else
-describe_gopher(banner, ZeGopher)
+                                 
 #endif
-  char *banner;
-  GopherStruct *ZeGopher;
+               
+                         
 {
      char *tmpfilename;
      FILE *tmpfile;
@@ -2975,7 +3017,7 @@ format_description:
 
      GStoLink(ZeGopher, fileno(tmpfile), TRUE);
 
-     fprintf(tmpfile, "<URL:%s>\n\n", GSgetURL(ZeGopher));
+     fprintf(tmpfile, "<URL:%s>\n\n", GSgetURL(ZeGopher,""));
      fflush(tmpfile);
 
      if (GSisGplus(ZeGopher)) {
@@ -3025,7 +3067,7 @@ format_description:
 		    fprintf(tmpfile, "%s", cp);
 	       else
 		    cp = VIgetLang(ZeView);
-	       for (i=strlen(cp); i<14; i++)
+	       for (i=(cp?strlen(cp):0); i<14; i++)
 		    fputc(' ', tmpfile);
 
 	       cp = VIgetType(ZeView);
