@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.36
- * 1994/10/21 04:41:07
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopher/gopherrc.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.38 $
+ * $Date: 1995/11/03 21:18:19 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopher/gopherrc.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,14 @@
  * Utilities to read and write the .gopherrc file 
  *********************************************************************
  * Revision History:
- * gopherrc.c,v
+ * $Log: gopherrc.c,v $
+ * Revision 3.38  1995/11/03  21:18:19  lindner
+ * ANSIfication
+ *
+ * Revision 3.37  1995/03/09  05:23:52  lindner
+ * F.Macrides 03-Mar-1995: Ensure that files are closed on errors in
+ * RCisGlobalNew(), else the user's RC file updates will fail on exit.
+ *
  * Revision 3.36  1994/10/21  04:41:07  lindner
  * Add ANSI attached printer code..
  *
@@ -207,7 +214,7 @@ char *getenv();
  */
 
 RCMapObj *
-RCMnew()
+RCMnew(void)
 {
      RCMapObj *rcm;
 
@@ -221,8 +228,7 @@ RCMnew()
 }
 
 void
-RCMdestroy(rcm)
-  RCMapObj *rcm;
+RCMdestroy(RCMapObj *rcm)
 {
      STRdestroy(rcm->view);
      STRdestroy(rcm->displaycmd);
@@ -231,8 +237,7 @@ RCMdestroy(rcm)
 }
 
 void
-RCMinit(rcm)
-  RCMapObj *rcm;
+RCMinit(RCMapObj *rcm)
 {
      STRinit(rcm->view);
      STRinit(rcm->displaycmd);
@@ -241,8 +246,7 @@ RCMinit(rcm)
 
 	  
 void
-RCMcpy(dest, orig)    
-  RCMapObj *dest, *orig;
+RCMcpy(RCMapObj *dest, RCMapObj *orig)
 {
      STRcpy(dest->view, orig->view);
      STRcpy(dest->displaycmd, orig->displaycmd);
@@ -257,9 +261,7 @@ RCMcpy(dest, orig)
  */
 
 static void
-RCMsubst_percent_s(rcm, filename, line, zeline)
-  RCMapObj *rcm;
-  char *filename, *line, *zeline;
+RCMsubst_percent_s(RCMapObj *rcm, char *filename, char *line, char *zeline)
 {
      
      while (1) {
@@ -296,9 +298,7 @@ RCMsubst_percent_s(rcm, filename, line, zeline)
 
 
 void
-RCMdisplayCommand(rcm, filename, line)
-  RCMapObj *rcm;
-  char *filename, *line;
+RCMdisplayCommand(RCMapObj *rcm, char *filename, char *line)
 {
      char *zeline;
 
@@ -315,9 +315,7 @@ RCMdisplayCommand(rcm, filename, line)
 
 
 void
-RCMprintCommand(rcm, filename, line)
-  RCMapObj *rcm;
-  char *filename, *line;
+RCMprintCommand(RCMapObj *rcm, char *filename, char *line)
 {
      char *zeline;
 
@@ -332,9 +330,7 @@ RCMprintCommand(rcm, filename, line)
  */
 
 int
-RCMAviewSearch(rcma, view)
-  RCMAarray *rcma;
-  char      *view;
+RCMAviewSearch(RCMAarray *rcma, char *view)
 {
      int i;
      RCMapObj *temp;
@@ -366,9 +362,7 @@ RCMAviewSearch(rcma, view)
  */
 
 boolean
-RCMAfromLine(rcma, line)
-  RCMAarray *rcma;
-  char *line;
+RCMAfromLine(RCMAarray *rcma, char *line)
 {
      char *cp;
      RCMapObj *rcm;
@@ -414,9 +408,7 @@ RCMAfromLine(rcma, line)
  */
 
 void
-RCMAtoFile(rcma, fio)
-  RCMAarray *rcma;
-  FileIO *fio;
+RCMAtoFile(RCMAarray *rcma, FileIO *fio)
 {
      int i;
      RCMapObj *rcm;
@@ -442,7 +434,7 @@ static BOOLEAN SettingRCdefs = FALSE;
 
 
 RCobj *
-RCnew()
+RCnew(void)
 {
      RCobj *rc;
 
@@ -469,8 +461,7 @@ RCnew()
 
 
 static FileIO *
-rcopen(flags) 
-  int flags;
+rcopen(int flags)
 {
      char    rcfilename[MAXPATHLEN];
      FileIO *fio;
@@ -497,9 +488,7 @@ rcopen(flags)
 
 
 void
-RCfromFile(rc, rcfio)
-  RCobj  *rc;
-  FileIO *rcfio;
+RCfromFile(RCobj *rc, FileIO *rcfio)
 {
      char inputline[512];
 
@@ -562,8 +551,7 @@ RCfromFile(rc, rcfio)
  */
 
 void
-RCfromENV(rc)
-  RCobj *rc;
+RCfromENV(RCobj *rc)
 {
      char tmpstr[512], Printer[128], PrinterP[128], Pager[128], PagerP[128];
      boolean PrinterFlag = FALSE;
@@ -647,8 +635,7 @@ RCfromENV(rc)
  */
 
 void
-RCfromUser(rc)
-  RCobj *rc;
+RCfromUser(RCobj *rc)
 {
      FileIO *rcfile;
 #ifndef VMS
@@ -691,8 +678,7 @@ RCfromUser(rc)
 
 
 void
-RCreadGlobalRC(rc)
-  RCobj *rc;
+RCreadGlobalRC(RCobj *rc)
 {
      FileIO *rcfile;
 
@@ -716,8 +702,7 @@ RCreadGlobalRC(rc)
  */
 
 void
-RCsetdefs(rc)
-  RCobj *rc;
+RCsetdefs(RCobj *rc)
 {
      char    tmpstr[512];
 
@@ -749,8 +734,7 @@ RCsetdefs(rc)
 }     
 
 void
-RCtoFile(rc)
-  RCobj *rc;
+RCtoFile(RCobj *rc)
 {
      FileIO *rcfile;
      char   rcfilename[512];
@@ -848,9 +832,7 @@ RCtoFile(rc)
  */
 
 boolean 
-RCdisplayCommand(rc, view, filename, line)
-  RCobj *rc;
-  char  *view, *filename, *line;
+RCdisplayCommand(RCobj *rc, char *view, char *filename, char *line)
 {
      int num;  
 
@@ -872,9 +854,7 @@ RCdisplayCommand(rc, view, filename, line)
  */
 
 boolean 
-RCprintCommand(rc, view, filename, line)
-  RCobj *rc;
-  char  *view, *filename, *line;
+RCprintCommand(RCobj *rc, char *view, char *filename, char *line)
 {
      int num;  
 
@@ -894,7 +874,7 @@ RCprintCommand(rc, view, filename, line)
  */
 
 boolean
-RCisGlobalNew() 
+RCisGlobalNew(void)
 {
      FileIO *fiolocal;
      FileIO *fioglobal;
@@ -908,14 +888,22 @@ RCisGlobalNew()
 
      fioglobal = FIOopenUFS(GLOBALRC, O_RDONLY, 0644);
 
-     if (fioglobal == NULL)
+     if (fioglobal == NULL) {
+          FIOclose(fiolocal);
 	  return(FALSE);
+     }
 
-     if (fstat(FIOgetfd(fiolocal), &lbuf))
+     if (fstat(FIOgetfd(fiolocal), &lbuf)) {
+          FIOclose(fiolocal);
+          FIOclose(fioglobal);
 	  return(FALSE);
+     }
 
-     if (fstat(FIOgetfd(fioglobal), &gbuf))
+     if (fstat(FIOgetfd(fioglobal), &gbuf)) {
+          FIOclose(fiolocal);
+          FIOclose(fioglobal);
 	  return(FALSE);
+     }
      
      FIOclose(fioglobal);
      FIOclose(fiolocal);
