@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.71
- * 1994/12/03 01:55:43
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopher/ourutils.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.77 $
+ * $Date: 1995/11/03 21:18:21 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopher/ourutils.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,29 @@
  * stuff that doesn't really fit anywhere else.
  *********************************************************************
  * Revision History:
- * ourutils.c,v
+ * $Log: ourutils.c,v $
+ * Revision 3.77  1995/11/03  21:18:21  lindner
+ * ANSIfication
+ *
+ * Revision 3.76  1995/11/03  20:50:51  lindner
+ * Coen: fixes..
+ *
+ * Revision 3.75  1995/04/15  07:10:13  lindner
+ * New control C methods, CTRL-X mods
+ *
+ * Revision 3.74  1995/03/14  16:08:41  lindner
+ * TPU fix
+ *
+ * Revision 3.73  1995/03/09  05:27:23  lindner
+ * F.Macrides 08-Mar-1995: Use strncasecmp() for getting the AskLs value
+ * in AskBlock().
+ *
+ * F.Macrides 02-Mar-1995: Allow %s to be at the end of 'builtin' when
+ * defining a viewer.
+ *
+ * Revision 3.72  1995/01/25  23:10:07  lindner
+ * Remove hard limit of 5 AskL items per form
+ *
  * Revision 3.71  1994/12/03  01:55:43  lindner
  * JL Wilkinson 16-Jul-1994 Merged in TGV's syslog() functions to allow
  * the CLIENT_LOGGER and TELNET_TRACE functionality to work with
@@ -283,11 +305,10 @@ char *getenv();
 #include "Locale.h"
 #include "String.h"
 
-void GSprompter();
+void GSprompter(GopherObj *gs, char *view, char *command);
 
 int 
-outchar(c)
-  char c;
+outchar(char c)
 {
         /** output the given character.  From tputs... **/
         /** Note: this CANNOT be a macro!              **/
@@ -303,7 +324,7 @@ outchar(c)
  */
 
 int
-twirl()
+twirl(void)
 {
      static int twirlnum = 0;
      static char *twirls = "-/|\\";
@@ -322,8 +343,7 @@ twirl()
 */
 
 void
-GSmail(gs)
-  GopherObj *gs;
+GSmail(GopherObj *gs)
 {
      char        *Filename = GSgetLocalFile(gs);
      char        *Realname = GSgetTitle(gs);
@@ -332,7 +352,10 @@ GSmail(gs)
      char         command[512];
      FileIO      *infile, *mailit;
      char         buf[512];
+
+#ifdef VMS
      char         MailAddress[512];
+#endif
 
      
      if (SaveName==NULL) {
@@ -432,8 +455,7 @@ GSmail(gs)
 
 
 char *
-Choose_View(gs)
-  GopherObj *gs;
+Choose_View(GopherObj *gs)
 {
      int viewno;
      static char viewnlang[64];
@@ -558,7 +580,7 @@ Choose_View(gs)
 }
 
 Requestitem *
-REQitemnew() 
+REQitemnew(void)
 {
      Requestitem *temp;
 
@@ -575,8 +597,7 @@ REQitemnew()
 }
 
 void
-REQitemdestroy(req)
-  Requestitem *req;
+REQitemdestroy(Requestitem *req)
 {
      int j;
 
@@ -600,15 +621,14 @@ REQitemdestroy(req)
  */
 
 char **
-AskBlock(gs)
-  GopherObj *gs;
+AskBlock(GopherObj *gs)
 {
-     int Asknum, reqnum;
+     int Asknum, reqnum, AskLs = 0;
      Blockobj *bl;
      Requestitem **items;
      char **asktypes;
      char **responses = NULL;
-     char askline[256];
+     char askline[1024];
      char *defaultval;
      int i;
 
@@ -619,16 +639,26 @@ AskBlock(gs)
      if (bl == NULL)
 	  return(NULL);
 
+     /** Find out how many ASKL items we have... **/
+     for (Asknum=0; Asknum < BLgetNumLines(bl); Asknum++) {
+	  if (strncasecmp(BLgetLine(bl, Asknum), "AskL:", 5) == 0)
+	       AskLs++;
+     }
+
      /** Can't guess how many askl things we'll have.. **/
-     items = (Requestitem **) malloc(sizeof(Requestitem*)* (BLgetNumLines(bl)+50));
-     asktypes = (char **) malloc(sizeof(char*) * (BLgetNumLines(bl) + 50));
+     items = (Requestitem **) malloc(sizeof(Requestitem*)* 
+				     (BLgetNumLines(bl) + (AskLs * 10) + 1));
+
+     asktypes = (char **) malloc(sizeof(char*) * 
+				 (BLgetNumLines(bl) + (AskLs * 10) + 1));
 
      reqnum = 0;
 
      for (Asknum=0; Asknum <BLgetNumLines(bl); Asknum++, reqnum++) {
 	  char *askprompt, *cp;
 
-	  strcpy(askline, BLgetLine(bl, Asknum));
+	  strncpy(askline, BLgetLine(bl, Asknum), sizeof(askline));
+	  askline[sizeof(askline)-1] = '\0';
 	  items[reqnum] = REQitemnew();
 
 	  /*** find the type of question ***/
@@ -725,12 +755,11 @@ AskBlock(gs)
      
      if (!CURrequester(CursesScreen, GSgetTitle(gs), items)) {
 	  int respnum = 0;
-	  char *choosefile = NULL;
 
 	  responses = (char**) malloc(sizeof(char*)*(10+reqnum));
 
 	  for (i=0; i <reqnum; i++) {
-	       if (strcasecmp(asktypes[i], "AskL:")==0) {
+	       if (asktypes[i] && strcasecmp(asktypes[i], "AskL:")==0) {
 		    int alnum, extra;
 		    char numlinestr[16];
 
@@ -751,7 +780,7 @@ AskBlock(gs)
 		    }
 		    i += extra;
 
-	       } else if (strcasecmp(asktypes[i], "Select:")==0) {
+	       } else if (asktypes[i] && strcasecmp(asktypes[i], "Select:")==0) {
 		    if (items[i]->chooseitem == 0)
 			 responses[respnum++] = strdup("0");
 		    else
@@ -787,8 +816,7 @@ AskBlock(gs)
 #define GERRMSGSIZE 25
 
 void
-Gplus_Error(sockfd)
-  int sockfd;
+Gplus_Error(int sockfd)
 {
      char inputline[256];
      char *errmsg[GERRMSGSIZE];
@@ -831,11 +859,7 @@ Gplus_Error(sockfd)
  */
 
 boolean
-GStoFile(gs, f, view, twirlfn)
-  GopherObj *gs;
-  FILE *f;
-  char *view;
-  int (*twirlfn)();
+GStoFile(GopherObj *gs, FILE *f, char *view, int (*twirlfn) (/* ??? */))
 {
      int numread, sockfd;
      char buf[1024];
@@ -868,7 +892,7 @@ GStoFile(gs, f, view, twirlfn)
      if (GSisText(gs, view)) {
 	  char cso_click = '\0';
 
-	  while (readline(sockfd, buf, sizeof(buf)) > 0) {
+	  while (readline(sockfd, buf, sizeof(buf)) > 0 && !ControlCpressed) {
 	       ZapCRLF(buf);
 	       line ++;
 	       if (*buf == '.' && *(buf+1) == '\0')
@@ -903,7 +927,7 @@ GStoFile(gs, f, view, twirlfn)
 			      fputc('\n', f);
 			 }
 		    }
-	       }
+	       } /* CSO */
 	       else {
 		    fputs(buf, f);
 		    /** Don't cut long lines... **/
@@ -912,17 +936,24 @@ GStoFile(gs, f, view, twirlfn)
 	       }
 	       if ((line % 25) == 0)
 		    twirlfn();
+	  } /*** readline ***/
+	  if (ControlCpressed) {
+	       CursesErrorMsg("Interrupted by User");
+	       ControlCpressed = FALSE;
 	  }
      }
      else {
 	       
 #if defined(VMS) && defined(VMSRecords)
-          while((numread = readrecvbuf(sockfd, buf, 512)) > 0) {
+          while (((numread = readrecvbuf(sockfd, buf, 512)) > 0) &&
+		 !ControlCpressed) {
 	       if (numread < 512)
 		   for (i = numread; i < 512; i++)  buf[i] = 0;
                if (fwrite(buf, 512, 1, f) == 0) {
 #else
-	  while ((numread = readrecvbuf(sockfd, buf, sizeof buf)) > 0) {
+	  while (((numread = readrecvbuf(sockfd, buf, sizeof buf)) > 0) &&
+		 !ControlCpressed)
+	  {
 	       if (fwrite(buf, numread, 1, f) == 0) {
 #endif
 		    CursesErrorMsg(Gtxt("Problems Writing",123));
@@ -931,6 +962,13 @@ GStoFile(gs, f, view, twirlfn)
 	       }
 	       twirlfn();
 	  }
+ 	  if (ControlCpressed) {
+	       CursesErrorMsg("Interrupted by User");
+	       ControlCpressed = FALSE;
+	       closenet(sockfd);
+	       return(FALSE);
+	  }
+
      }
 
      if (GSgetType(gs) == A_CSO)
@@ -946,8 +984,7 @@ GStoFile(gs, f, view, twirlfn)
  */
 
 void
-VMSfile(fname)
-  char *fname;
+VMSfile(char *fname)
 {
 #ifndef VMS
      return;
@@ -1099,8 +1136,7 @@ VMSfile(fname)
  */
 
 void
-UNIXfile(fname)
-  char *fname;
+UNIXfile(char *fname)
 {
      char *cp;
 
@@ -1157,9 +1193,7 @@ UNIXfile(fname)
  */
 
 void
-Save_file(gs, saveto, view)
-  GopherObj *gs;
-  char *saveto, *view;
+Save_file(GopherObj *gs, char *saveto, char *view)
 {
      char    Title[128], Userfilename[128];
      char    *cp;
@@ -1301,11 +1335,17 @@ Save_file(gs, saveto, view)
 	       return;
 	  }
 	  
-	  while ( (cc=read(oldfd, buf, sizeof buf)) > 0) {
+	  while ( ((cc=read(oldfd, buf, sizeof buf)) > 0) && !ControlCpressed){
 	       if (fwrite(buf, cc, 1, f) <= 0)
 		    CursesErrorMsg(Gtxt("Problems Writing",123));
 	       twirl();
 	  }
+	  if (ControlCpressed) {
+	       CursesErrorMsg("Save Interrupted by user");
+	       ControlCpressed = FALSE;
+ 	  }
+ 
+
 	  if (Openpipe)
 	       pclose(f);
 	  else
@@ -1341,8 +1381,7 @@ Save_file(gs, saveto, view)
  */
 
 void
-Save_list(gd)
-  GopherDirObj *gd;
+Save_list(GopherDirObj *gd)
 {
      char    Title[256], Userfilename[128];
      char    *cp, *saveto;
@@ -1454,11 +1493,9 @@ Save_list(gd)
 */
 
 void
-GSdisplay(gs)
-  GopherObj *gs;
+GSdisplay(GopherObj *gs)
 {
      char command[MAXSTR];
-     int ch;
      char *file = GSgetLocalFile(gs);
      char *view = GSgetLocalView(gs);
 
@@ -1512,6 +1549,9 @@ GSdisplay(gs)
 	       CursesErrorMsg(Gtxt("Error invoking callable TPU as a pager!!!",85));
 	       return;
 	  }
+	  if (GSgetType(gs) != A_APP) {
+	       GSprompter(gs, view, command);
+	  }
      } else
 #endif
      if (strncasecmp(command, "builtin-upload", 14) == 0) {
@@ -1522,7 +1562,7 @@ GSdisplay(gs)
 	  BuiltinDownload(getenv("HOME"));
 	  return;
      }
-     else if (strcasecmp(command, "builtin") == 0) {
+     else if (strncasecmp(command, "builtin", 7) == 0) {
 	  PagerBuiltin(CursesScreen, gs);
 	  return;
      }
@@ -1542,9 +1582,7 @@ GSdisplay(gs)
  */
 
 void
-GSprompter(gs, view, command)
-  GopherObj *gs;
-  char *view, *command;
+GSprompter(GopherObj *gs, char *view, char *command)
 {
      int ch;
 
@@ -1709,9 +1747,7 @@ GSprompter(gs, view, command)
  */
      
 void
-CursesMsg(Message, title)
-  char *Message;
-  char *title;
+CursesMsg(char *Message, char *title)
 {
      char *mess[2];
      
@@ -1733,8 +1769,7 @@ CursesMsg(Message, title)
  */
 
 void
-CursesErrorMsg(Message)
-  char *Message;
+CursesErrorMsg(char *Message)
 {
      CursesMsg(Message, Gtxt("Gopher Error",90));
 }
@@ -1750,9 +1785,7 @@ CursesErrorMsg(Message)
 #endif
 
 int
-logrequest(msg, gs)
-  char *msg;
-  GopherObj *gs;
+logrequest(char *msg, GopherObj *gs)
 {
 #ifdef CLIENT_LOGGER
      static boolean inited = FALSE;
@@ -1764,7 +1797,7 @@ logrequest(msg, gs)
 	  inited = TRUE;
      }
      if (gs != NULL)
-	  url = GSgetURL(gs);
+	  url = GSgetURL(gs, "");
 
      syslog(LOG_INFO, "%s %s", msg, url ? url : "");
 
