@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.28
- * 1994/12/03 01:52:19
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopher/pager.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.34 $
+ * $Date: 1995/11/03 21:18:22 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopher/pager.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,25 @@
  * neato pager..
  *********************************************************************
  * Revision History:
- * pager.c,v
+ * $Log: pager.c,v $
+ * Revision 3.34  1995/11/03  21:18:22  lindner
+ * ANSIfication
+ *
+ * Revision 3.33  1995/05/01  05:41:30  lindner
+ * compatibility fixes
+ *
+ * Revision 3.32  1995/05/01  03:41:49  lindner
+ * Fix return value
+ *
+ * Revision 3.31  1995/04/15  07:08:03  lindner
+ * Fix
+ *
+ * Revision 3.30  1995/02/23  21:53:53  lindner
+ * Correct cast..
+ *
+ * Revision 3.29  1995/01/03  19:53:25  lindner
+ * Dynamic screen position code, long title fix from Allan L. Bazinet
+ *
  * Revision 3.28  1994/12/03  01:52:19  lindner
  * Don't auto exit paging at the end of the file
  *
@@ -64,7 +82,14 @@
 #include "fileio.h"
 #include "Malloc.h"
 
-#include "Stat.h"
+#ifdef VMS
+#include <stat.h>
+#else
+#include <sys/stat.h>
+#endif
+
+
+#define PosIncrement 10      /* room for 10 screens per malloc */
 
 /** Argv/argc for searched words.. **/
 static char *words[50];   
@@ -74,9 +99,10 @@ static int  wordcount = 0;
 static char slashstring[128];
 
 /** bytecount positions for each page..  (non-dynamic.. yet...) **/
-static int  positions[4000];  /** Bad programmer **/
+/* static int  positions[4000]; */  /** Bad programmer **/
+static int *positions = NULL;  /* not-so-bad programmer */
 static int  currentpage;
-
+static int  pos_max;
 
 /*
  * This is the beginning of a built in pager..  
@@ -85,8 +111,7 @@ static int  currentpage;
  */
 
 void
-PagerHelp(cur)
-  CursesObj *cur;
+PagerHelp(CursesObj *cur)
 {
      static char **helpmenu = NULL;
 
@@ -107,7 +132,7 @@ PagerHelp(cur)
 }
 
 void
-PagerInitGlobals()
+PagerInitGlobals(void)
 {
      currentpage = 0;
      wordcount = 0;
@@ -116,22 +141,31 @@ PagerInitGlobals()
 
 
 void
-PagerTitles(cur, gs, totalbytes)
-  CursesObj *cur;
-  GopherObj *gs;
-  int totalbytes;
+PagerTitles(CursesObj *cur, GopherObj *gs, int totalbytes)
 {
-     int i;
+     int i, fluff_len, k_bytes;
+     char *cp;
+
+     k_bytes = totalbytes/1024;
 
      /* Okay, first draw the top... ***/
      
      move(0,0);
      wboldout(stdscr);
-     waddstr(stdscr, GSgetTitle(gs));
+
+     fluff_len = 11 + (k_bytes/10);     /* 11 = strlen("_(0k)_100%_"); */
+     if((strlen(GSgetTitle(gs))) <= (COLS-fluff_len)) {
+       waddstr(stdscr, GSgetTitle(gs));
+     } else {
+       cp = GSgetTitle(gs);
+       for(i = 0; i < (COLS-(fluff_len+3)); i++) {
+	 waddch(stdscr, *cp++);
+       }
+       waddstr(stdscr, "...");
+     }
      wboldend(stdscr);
 
-     wprintw(stdscr, " (%dk)", totalbytes/1024);
-
+     wprintw(stdscr, " (%dk)", k_bytes);
 
      wmove(stdscr, 1,0);
      waddch(stdscr, CURgetBox_ul(cur));
@@ -159,11 +193,47 @@ PagerTitles(cur, gs, totalbytes)
      CURbutton(cur, stdscr, Gtxt("Return to Menu: u",134), FALSE);
 }
 
+int PagePosAlloc(void)
+{
+  size_t MallocSize;
+
+  MallocSize = sizeof(int) * PosIncrement;
+  positions = (int *) malloc(MallocSize);
+  if (!positions)
+    return(FALSE);
+  else {
+    pos_max = PosIncrement;
+    return(TRUE);
+  }
+}
+
+
+int PagePosRealloc(void)
+{
+  int *temp = NULL;  /* in case realloc fails */
+  size_t MallocSize;
+
+  MallocSize = sizeof(int) * (pos_max + PosIncrement);
+  temp = positions;
+  positions = (int *) realloc(positions, MallocSize);
+  if(!positions) {
+    positions = temp;
+    return(FALSE);
+  } else {
+    pos_max += PosIncrement;
+    return(TRUE);
+  }
+}
+
+
+void PagePosFree(void)
+{
+  if(positions) free(positions);
+}
+
 
 void
-PagerPercent(bytes, totalbytes)
-  int bytes;
-  int totalbytes;
+PagerPercent(int bytes, int totalbytes)
 {
      int per = (totalbytes? ((100 * bytes)/totalbytes) : 0);
      
@@ -181,7 +251,7 @@ PagerPercent(bytes, totalbytes)
  */
 
 void
-PagerParseSearchstring()
+PagerParseSearchstring(void)
 {
      char *MungeSearchstr = Searchstring;
      int  numchars;
@@ -222,8 +292,7 @@ PagerParseSearchstring()
 
 
 void
-PagerPrintLine(inputline)
-  char *inputline;
+PagerPrintLine(char *inputline)
 {
      int lowwordnum = -1, i;
      char *cp, *lowword;
@@ -294,10 +363,7 @@ PagerPrintLine(inputline)
 
 
 void
-PagerSeekPage(thefile, gotopage, bytecount)
-  FILE *thefile;
-  int  gotopage;
-  int  *bytecount;
+PagerSeekPage(FILE *thefile, int gotopage, int *bytecount)
 {
      if (gotopage < currentpage) {
 	  fseek(thefile, positions[gotopage], 0);
@@ -308,12 +374,7 @@ PagerSeekPage(thefile, gotopage, bytecount)
 
 
 void
-PagerNextPage(cur, thefile, theline, bytecount, totalbytes)
-  CursesObj *cur;
-  FILE      *thefile;
-  char      *theline;
-  int       *bytecount;
-  int       totalbytes;
+PagerNextPage(CursesObj *cur, FILE *thefile, char *theline, int *bytecount, int totalbytes)
 {
      int i;
      char *cp = NULL;
@@ -323,6 +384,11 @@ PagerNextPage(cur, thefile, theline, bytecount, totalbytes)
 
      /** Save current bytecount position **/
      positions[currentpage++] = *bytecount;
+     if(currentpage == pos_max)
+       if(!PagePosRealloc()) {
+	 CursesErrorMsg(Gtxt("Sorry, can't display this file",148));
+	 return;
+       }
 
      wmove(stdscr, 2, 0);
 
@@ -379,11 +445,7 @@ PagerNextPage(cur, thefile, theline, bytecount, totalbytes)
  */
 
 boolean
-PagerSearch(thefile, theline, bytecount, search)
-  FILE *thefile;
-  char *theline;
-  int  *bytecount;
-  char *search;
+PagerSearch(FILE *thefile, char *theline, int *bytecount, char *search)
 {
      int i;
      char *cp = NULL;
@@ -395,8 +457,11 @@ PagerSearch(thefile, theline, bytecount, search)
      do {
 	  /** Save current bytecount position **/
 	  positions[currentpage++] = *bytecount;
-
-	  
+	  if(currentpage == pos_max)
+	    if(!PagePosRealloc()) {
+	      CursesErrorMsg(Gtxt("Sorry, can't display this file",148));
+	      return(FALSE);
+	    }
 	  for (i=0; i<LINES-4; i++) {
 	  search_again:
 	       cp = fgets(theline, COLS+1, thefile);
@@ -425,9 +490,7 @@ PagerSearch(thefile, theline, bytecount, search)
 }
 
 void
-PagerBuiltin(cur, gs)
-  CursesObj *cur;
-  GopherObj *gs;
+PagerBuiltin(CursesObj *cur, GopherObj *gs)
 {
      int         ch;
      FILE        *thefile;
@@ -442,6 +505,11 @@ PagerBuiltin(cur, gs)
 
      PagerInitGlobals();
      
+     if(!PagePosAlloc()) {
+       CursesErrorMsg(Gtxt("Sorry, can't display this file",148));
+       return;
+     }
+
      /*** Initialize an array for the screen ... ***/
      theline = (char *) malloc(sizeof(char)*(COLS+1));
      *theline = '\0';
@@ -459,6 +527,7 @@ PagerBuiltin(cur, gs)
 
      if (thefile == NULL) {
 	  CursesErrorMsg(Gtxt("Cannot Open requested file..",71));
+	  if(theline) free(theline);
 	  return;
      }
 
@@ -685,6 +754,7 @@ PagerBuiltin(cur, gs)
 
      fclose(thefile);
      free(theline);
+     PagePosFree();
 }
 
 
