@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.41
- * 1994/11/18 21:41:14
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopherd/ftp.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.52 $
+ * $Date: 1995/11/03 18:13:58 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopherd/ftp.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,40 @@
  * Routines to translate gopher protocol to ftp protocol.
  *********************************************************************
  * Revision History:
- * ftp.c,v
+ * $Log: ftp.c,v $
+ * Revision 3.52  1995/11/03  18:13:58  lindner
+ * Coen: ftp gw fixes..
+ *
+ * Revision 3.51  1995/09/28  21:11:29  lindner
+ * More fixes...
+ *
+ * Revision 3.50  1995/09/26  05:00:46  lindner
+ * Minor fix..
+ *
+ * Revision 3.49  1995/09/26  04:59:44  lindner
+ * Fix abort routines, change to Die/Warn hash over access routines..
+ *
+ * Revision 3.48  1995/09/25  22:06:56  lindner
+ * Ansification bugs
+ *
+ * Revision 3.47  1995/09/25  05:02:33  lindner
+ * Convert to ANSI C
+ *
+ * Revision 3.46  1995/09/09  03:20:06  lindner
+ * FTP username and password support
+ *
+ * Revision 3.45  1995/08/29  07:10:49  lindner
+ * URL code changes..
+ *
+ * Revision 3.44  1995/07/29  04:39:03  lindner
+ * Add URL ftp support for the server
+ *
+ * Revision 3.43  1995/02/20  23:36:50  lindner
+ * Fix for searches
+ *
+ * Revision 3.42  1995/02/06  21:24:42  lindner
+ * Fix for pedantic compilers
+ *
  * Revision 3.41  1994/11/18  21:41:14  lindner
  * Remove unused variable, memory leak
  *
@@ -177,6 +210,7 @@
 
 #include "gopherd.h"
 #include "ftp.h"
+#include "url.h"
 #include <signal.h>
 
 #include <stdio.h>
@@ -187,7 +221,6 @@
 
 #include "Debug.h"
 
-#define SLEN 256
 
 
 /** Global socket value **/
@@ -199,19 +232,15 @@ static int GFTPplus;
 /** track whether we sent a Gplus header already **/
 static int SentGPHeader;
 
-boolean GETDIR = FALSE;
+static boolean GETDIR = FALSE;
 
 /*** Some forward declarations ***/
-boolean NotText();
-boolean IsBinaryType();
-void    Cleanup();
-void    FailErr();
-char    *WalkDirs();
-boolean FTPconnect();
+boolean NotText(char *);
+
+boolean FTPconnect(FTP *ftp, char *host);
 
 FTP *
-FTPnew(host)
-  char *host;
+FTPnew(char *host)
 {
      FTP *temp;
 
@@ -242,8 +271,7 @@ FTPnew(host)
  */
 
 void
-FTPdestroy(ftp)
-  FTP *ftp;
+FTPdestroy(FTP *ftp)
 {
      FTPcloseData(ftp);
 
@@ -263,9 +291,7 @@ FTPdestroy(ftp)
 */
 
 boolean
-FTPconnect(ftp, host)
-  FTP  *ftp;
-  char *host;
+FTPconnect(FTP *ftp, char *host)
 {
      int newcontrol;
      char message[256];
@@ -324,8 +350,7 @@ FTPconnect(ftp, host)
 
 
 char *
-FTPpwd(ftp)
-  FTP *ftp;
+FTPpwd(FTP *ftp)
 {
      static char pwd[256];
      char message[256];
@@ -344,8 +369,7 @@ FTPpwd(ftp)
 
 
 void
-FTPfindType(ftp)
-  FTP *ftp;
+FTPfindType(FTP *ftp)
 {
      int ftpnum;
      char message[256];
@@ -382,9 +406,7 @@ FTPfindType(ftp)
    Returns true if an error, false otherwise
 */
 boolean
-FTPsend(ftp, command)
-  FTP  *ftp;
-  char *command;
+FTPsend(FTP  *ftp, char *command)
 {
      Debug("--> %s\n", command);
 
@@ -401,26 +423,17 @@ FTPsend(ftp, command)
 /* Find the numeric value of the message */
 
 int
-FTPfindNum(ftp, message)
-  FTP  *ftp;
-  char *message;
+FTPfindNum(FTP *ftp, char *message)
 {
      int ftpnum;
 
-/*
-     ftpnum = (message[0] - '0') * 100 +
-	      (message[1] - '0') * 10 +
-	      (message[2] - '0');
-*/
-	 ftpnum = (int) strtol(message, (char **)NULL, 10);
+     ftpnum = (int) strtol(message, (char **)NULL, 10);
      return(ftpnum);
 }
 
 
 boolean
-FTPisContinuation(ftp, message)
-  FTP  *ftp;
-  char *message;
+FTPisContinuation(FTP *ftp, char *message)
 {
 	if (message[0] == '\0')
 		return TRUE;
@@ -436,11 +449,7 @@ FTPisContinuation(ftp, message)
  */
 
 int
-FTPgetReply(ftp, errorlevel, message, maxlen)
-  FTP  *ftp;
-  int  errorlevel;
-  char *message;
-  int  maxlen;
+FTPgetReply(FTP  *ftp, int  errorlevel, char *message, int  maxlen)
 {
      int ftpnum;
 
@@ -489,10 +498,7 @@ FTPgetReply(ftp, errorlevel, message, maxlen)
  */
 
 int
-FTPgetReplyline(ftp, message, maxlen)
-  FTP  *ftp;
-  char *message;
-  int  maxlen;
+FTPgetReplyline(FTP  *ftp, char *message, int  maxlen)
 {
      int ftpnum, i;
 
@@ -519,11 +525,7 @@ FTPgetReplyline(ftp, message, maxlen)
  */
 
 int
-FTPcommand(ftp, command, parameter, errorlevel)
-  FTP  *ftp;
-  char *command;
-  char *parameter;
-  int  errorlevel;
+FTPcommand(FTP  *ftp, char *command, char *parameter, int  errorlevel)
 {
      char commandline[512];
      char message[256];
@@ -541,10 +543,7 @@ FTPcommand(ftp, command, parameter, errorlevel)
  */
 
 int
-FTPlogin(ftp, username, password)
-  FTP  *ftp;
-  char *username;
-  char *password;
+FTPlogin(FTP  *ftp, char *username, char *password)
 {
      int result;
 
@@ -563,8 +562,7 @@ FTPlogin(ftp, username, password)
 
 
 void
-FTPbyebye(ftp)
-  FTP *ftp;
+FTPbyebye(FTP *ftp)
 {
      FTPcommand(ftp, "QUIT", NULL, 399);
 }
@@ -572,11 +570,7 @@ FTPbyebye(ftp)
 
 /** Open a data connection **/
 void
-FTPopenData(ftp, command, file, errorlevel)
-  FTP *ftp;
-  char *command;
-  char *file;
-  int errorlevel;
+FTPopenData(FTP *ftp, char *command, char *file, int errorlevel)
 {
      struct sockaddr_in we;
      char   theline[512];
@@ -610,18 +604,14 @@ FTPopenData(ftp, command, file, errorlevel)
 
 
 void
-FTPcloseData(ftp)
-  FTP *ftp;
+FTPcloseData(FTP *ftp)
 {
      if (FTPgetData(ftp) != -1)
 	  close(FTPgetData(ftp));
 }
 
 int
-FTPread(ftp, buf, bufsize)
-  FTP *ftp;
-  char *buf;
-  int bufsize;
+FTPread(FTP *ftp, char *buf, int bufsize)
 {
      int len;
 
@@ -634,8 +624,7 @@ FTPread(ftp, buf, bufsize)
 /*--------------------------------*/
 /* Used in xLateText below --------*/
 boolean
-NotText(buf)
-  char * buf;
+NotText(char *buf)
 {
      int max;   char *c;
      
@@ -654,29 +643,36 @@ static char *textnames[] =
 /* return false if extension indicates text, true if binary */
 
 boolean
-IsBinaryType(path)
-  char *path;
+IsBinaryType(char *path)
 {
      char Gtype;
      Extobj *ext;
      char *slashp, **textn, *cp;
      
+     ext = EXnew();
+
+
      if (GDCViewExtension(Config, path, &ext)) {
 	  Gtype = EXgetObjtype(ext);
 	  
 	  switch (Gtype) {
+
 	  case A_FILE:
 	  case A_MACHEX:
 	  case A_CSO:
 	  case A_INDEX:
 	  case A_TELNET:
 	  case A_TN3270:
+	       EXdestroy(ext);
 	       return FALSE;
 
-/*	  default:
-	       return TRUE;*/
 	  }
      }
+     if (EXAcasedSearch(Config->Extensions, ext, path, EXT_DECODER)) {
+	  EXdestroy(ext);
+	  return TRUE;
+     }
+
      if ((slashp = strrchr(path, '/')) == NULL)
 	  slashp = path;
      else
@@ -698,8 +694,7 @@ IsBinaryType(path)
 
 /*--------------------------------*/
 void
-TrimEnd(bufptr)
-  char *bufptr;
+TrimEnd(char *bufptr)
 {
      int last;
      for (last = strlen(bufptr) - 1; isspace(bufptr[last]) ; bufptr[last--] = '\0')
@@ -709,8 +704,7 @@ TrimEnd(bufptr)
 
 
 int
-Vaxinate(bufptr)
-  char *bufptr;
+Vaxinate(char *bufptr)
 {
      int last;
      char *cp;
@@ -734,7 +728,7 @@ Vaxinate(bufptr)
      }
 
      /* for files, uppercase terminal .z's or _z's */
-     if (strlen(bufptr) > 1) {
+     if ((int)strlen(bufptr) > 1) {
           if (bufptr[last] == 'z' &&
 	     (bufptr[last-1] == '.' || bufptr[last-1] == '_'))
                bufptr[last] = 'Z';
@@ -744,9 +738,7 @@ Vaxinate(bufptr)
 }
 
 void
-FTPchdir(ftp, path)
-  FTP  *ftp;
-  char *path;
+FTPchdir(FTP *ftp, char *path)
 {
      int result;
      char *ptr;
@@ -762,6 +754,7 @@ FTPchdir(ftp, path)
 		    ptr++;
 	       *ptr = '\0';
 	       FTPchdir(ftp, path);
+	       return;
 	  }
 	  /* else fall through */
      default:
@@ -774,9 +767,7 @@ FTPchdir(ftp, path)
 
 
 char *
-FTPwalkchdir(ftp, path)
-  FTP  *ftp;
-  char *path;
+FTPwalkchdir(FTP  *ftp, char *path)
 {
      char *beg, *end, *cp;
      char vmspath[256], tmppath[256];
@@ -828,9 +819,7 @@ FTPcleanup()
 }
 
 int
-FTPerrorMessage(ftp, message)
-  FTP *ftp;
-  char *message;
+FTPerrorMessage(FTP *ftp, char *message)
 {
      char errmsg[256];
 	 char *cp;
@@ -857,9 +846,7 @@ FTPerrorMessage(ftp, message)
 }
 
 int
-FTPinfoMessage(ftp, message)
-  FTP *ftp;
-  char *message;
+FTPinfoMessage(FTP *ftp, char *message)
 {
      char errmsg[512];
      char *cp;
@@ -875,7 +862,7 @@ FTPinfoMessage(ftp, message)
      ZapCRLF(message);
 
      if (GETDIR) {
-	  if (strlen(message) > 3 && message[3] == '-')
+	  if ((int)strlen(message) > 3 && message[3] == '-')
 	       sprintf(errmsg, "i%s\t\terror.host\t1\r\n", &message[4]);
 	  else
 	       sprintf(errmsg, "i%s\t\terror.host\t1\r\n", message);
@@ -894,28 +881,26 @@ FTPinfoMessage(ftp, message)
 }
 
 void
-FTPabort(ftp)
-  FTP *ftp;
+FTPabort(FTP *ftp)
 {
      writestring(Gsockfd, ".\r\n");
      FTPdestroy(ftp);
 }
 
 
+Url *ftpurl;
+
 /*
  * Implement ftp--> gopher gateway
  */
 
 int
-GopherFTPgw(sockfd, ftpstr, cmd)
-  int    sockfd;
-  char   *ftpstr;
-  CMDobj *cmd;
+GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 {
-     FTP  *ftp;
-     char *cp;
-     char ftp_info_fname[256];
-     char *ftphost, *ftpuser, ftppass[256], *ftppath;
+     FTP    *ftp;
+     char   *cp;
+     char   ftp_info_fname[256];
+     char   *ftphost, *ftpuser=NULL, ftppass[256], *ftppath;
      char buf[8192];
      int blen;
      char lastchar;
@@ -926,21 +911,60 @@ GopherFTPgw(sockfd, ftpstr, cmd)
      Gsockfd = sockfd;
      
      /* Find @ and parse out */
-     cp = strchr(ftpstr, '@');
-     if (cp == NULL)
-	  return(-1);
+     if (strncmp(ftpstr, "//", 2) == 0) {
+	  char *mystr;
+	  /** Must be a url stylee string... **/
+	  mystr  = (char*)malloc(strlen(ftpstr) + 5);
+	  strcpy(mystr, "ftp:");
+	  strcat(mystr, ftpstr);
+	  ftpurl = URLnew();
+	  URLset(ftpurl, mystr);
 
-     *cp = '\0';
-     
-     ftppath = cp+1;
-     ftphost = ftpstr;
+	  if (URLgetUser(ftpurl) != NULL)
+	       ftpuser = strdup(URLgetUser(ftpurl));
+	  if (URLgetPass(ftpurl) != NULL)
+	       strcpy(ftppass, URLgetPass(ftpurl));
+	  ftppath = strdup(URLgetPath(ftpurl));
+	  ftphost = strdup(URLgetHost(ftpurl));
+
+	  if (strlen(ftppath) == 0) {
+	       ftppath[0] = '/';
+	       ftppath[1] = '\0';
+	  }
+	  free(mystr);
+	  
+     } else {
+	  cp = strchr(ftpstr, '@');
+	  if (!cp)
+	       return(-1);
+
+	  *cp = '\0';
+	  
+	  ftppath = cp+1;
+	  ftphost = ftpstr;
+     }
+
+     if (!ftpuser)
+	  ftpuser = "anonymous";
 
-     ftpuser = "anonymous";
      Gsockfd = sockfd;
-     GFTPplus = CMDisGplus(cmd);
+/*     GFTPplus = CMDisGplus(cmd);*/
+     GFTPplus = FALSE;
      SentGPHeader = FALSE;
 
-     sprintf(ftppass, "-gopher@%s", Zehostname);
+     if (*ftppath == '/') 
+	  ftppath++;
+ 
+     if (*ftppath == '\0') {
+	  *ftppath = '/';
+ 	  *(ftppath+1) = '\0';
+     }
+ 
+
+
+     if (*ftppass == '\0') {
+	  sprintf(ftppass, "-gopher@%s", Zehostname);
+     }
      if (Gopherstow == NULL)
 	  Gopherstow = GSnew();
 
@@ -955,13 +979,16 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 	  char tmpstr[256];
 	  
 	  lastslash = strrchr(ftppath, '/');
-	  if (lastslash == NULL) {
-	       Abortoutput(sockfd, ftppath);
-	       return(-1);
-	  }
+	  if (lastslash == NULL)
+	       Die(sockfd, 500, "Can't parse %s", ftppath);
+
 	  GSsetHost(gs, Zehostname);
 	  GSsetPort(gs, GopherPort);
-	  sprintf(tmpstr, "ftp:%s@%s", ftphost, ftppath);
+	  if (strcmp(ftpuser, "anonymous") != 0) {
+ 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), ftppath);
+	  } else {
+	       sprintf(tmpstr, "ftp:%s@%s", ftphost, ftppath);
+	  }
 	  GSsetPath(gs, tmpstr);
 	  GSsetTitle(gs, tmpstr);
 
@@ -1003,7 +1030,7 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 	  /*** Do the directory ***/
 	  ftppath[strlen(ftppath)-1] = '\0';
 
-	  if (strlen(ftppath) > 1) {	  
+	  if ((int)strlen(ftppath) > 1) {	  
 	       if (FTPgetType(ftp) == FTP_VMS 
 		   || FTPgetType(ftp) == FTP_NOVELL
 		   || FTPgetType(ftp) == FTP_MACOS
@@ -1038,8 +1065,12 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 	       FTPopenData(ftp, "LIST -LF", NULL, 299);
 	       break;
 
-	  case FTP_NOVELL:
+
 	  case FTP_UNIX_L8:
+	       FTPopenData(ftp, "LIST -F", NULL, 299);
+	       break;
+
+	  case FTP_NOVELL:
 	  case FTP_MICRO_VAX:
 	  case FTP_VMS:
 	  case FTP_VM:
@@ -1048,6 +1079,7 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 	       FTPopenData(ftp, "LIST", NULL, 299);
 	       break;
 
+
 	  default:
 	       FTPopenData(ftp, "NLST", NULL, 299);
 	       break;
@@ -1061,17 +1093,17 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 
 	       ZapCRLF(buf);
 	       GSinit(gs);
-	       GSsetGplus(gs, TRUE);
+	       GSsetGplus(gs, FALSE);
 	       GSsetHost(gs, ftphost);
 	       GSsetPath(gs, ftppath);
 	       GSsetPort(gs, GopherPort);
 	       GSsetTTL(gs, GDCgetCachetime(Config));
 	       if (GFTPplus)
 		    GSsetGplus(gs, TRUE);
-	       if (GopherList(ftp, buf, thename, gs) == -1)
+	       if (GopherList(ftp, buf, thename, gs, ftpuser, ftppass) == -1)
 		    continue;
 	       GSsetHost(gs, Zehostname);
-	       GSgetURL(gs);
+	       GSgetURL(gs,"");
 	       GDaddGS(gd, gs);
 	  }
 
@@ -1082,12 +1114,12 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 			 GSplustoNet(GDgetEntry(gd, fnum), sockfd, NULL, "");
 	       } else
 	       if (strncasecmp(CMDgetView(cmd), "text/html", 9) == 0) {
-		    GDtoNet(gd, sockfd, GSFORM_HTML, "");
+		    GDtoNet(gd, sockfd, GSFORM_HTML, "", NULL);
 	       }
 	       else
 		    GDplustoNet(gd, sockfd, NULL, "");
 	  } else
-	       GDtoNet(gd, sockfd, GSFORM_G0, "");
+	       GDtoNet(gd, sockfd, GSFORM_G0, "", NULL);
 	  writestring(sockfd, ".\r\n");
      }
      else {
@@ -1156,11 +1188,8 @@ GopherFTPgw(sockfd, ftpstr, cmd)
 ******************************/
 
 int
-GopherList(ftp, bufptr, theName, gs) 
-  FTP  *ftp;
-  char *bufptr;
-  char *theName;
-  GopherObj *gs;
+GopherList(FTP *ftp, char *bufptr, char *theName, GopherObj *gs,
+	   char *ftpuser, char *ftppass)
 {
      char Gzerotype;
      static char *IntName = NULL;
@@ -1183,29 +1212,34 @@ GopherList(ftp, bufptr, theName, gs)
      {
      case FTP_MACOS:
 	  Debugmsg("Parsing MACOS List\n");
-	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 6, gs);
+	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 6, gs,
+				    ftpuser, ftppass);
 	  break;
 
      case FTP_VMS:
 	  Debugmsg("Parsing VMS List\n");
-	  Gzerotype = ParseVMSList(ftp, bufptr, IntName, theName, gs);
+	  Gzerotype = ParseVMSList(ftp, bufptr, IntName, theName, gs,
+				   ftpuser, ftppass);
 	  break;
 
      case FTP_NOVELL:
 	  Debugmsg("Parsing Novell List\n");
-	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 6, gs);
+	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 6, gs, 
+				    ftpuser, ftppass);
 	  break;
 
      case FTP_UNIX:
      case FTP_UNIX_L8:
      case FTP_WINNT:
 	  Debugmsg("Parsing Unix List\n");
-	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 7, gs);
+	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 7, gs,
+				    ftpuser, ftppass);
 	  break;
 
      case FTP_OS2:
 	  Debugmsg("Parsing OS/2 List\n");
- 	  Gzerotype = ParseOS2List(ftp, bufptr, IntName, theName, gs);
+ 	  Gzerotype = ParseOS2List(ftp, bufptr, IntName, theName, gs, 
+				   ftpuser, ftppass);
  	  break;
 
 
@@ -1213,7 +1247,8 @@ GopherList(ftp, bufptr, theName, gs)
      case FTP_UNKNOWN:
      default:
 	  Debugmsg("Parsing Unknown List\n");
-	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 7, gs);
+	  Gzerotype = ParseUnixList(ftp, bufptr, IntName, theName, 7, gs,
+				    ftpuser, ftppass);
 	  break;
      }
 
@@ -1223,13 +1258,8 @@ GopherList(ftp, bufptr, theName, gs)
 
 
 int
-ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
-  FTP  *ftp;
-  char *bufptr;
-  char *IntName;
-  char *theName;
-  int cols;
-  GopherObj *gs;
+ParseUnixList(FTP *ftp, char *bufptr, char *IntName, char *theName,
+	      int cols, GopherObj *gs, char *ftpuser, char **ftppass)
 {
      int i, end;
      int gap, objsize;
@@ -1283,23 +1313,27 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
      
      if (gap < cols) 
 	  FTPerrorMessage(ftp, "said Unix but wasn't (short cols)");
-     
-     if (isdigit(bufptr[i]) && isdigit(bufptr[i+1])) {
-	  if (isdigit(bufptr[i+2]) && isdigit(bufptr[i+3]) &&
-	      isspace(bufptr[i+4])) { /* probably the YEAR */
-	       for (i+=4;isspace(bufptr[i]); i++);
-	  } else if ((bufptr[i+2] == ':') && isdigit(bufptr[i+3]) &&
-		     isdigit(bufptr[i+4]) && isspace(bufptr[i+5])) {
+
+     if (isdigit(bufptr[i])) {
+	  if (isdigit(bufptr[i+1])) {
+	       if (isdigit(bufptr[i+2]) && isdigit(bufptr[i+3]) &&
+		   isspace(bufptr[i+4])) { /* probably the YEAR */
+		    for (i+=4;isspace(bufptr[i]); i++);
+	       } else if ((bufptr[i+2] == ':') && isdigit(bufptr[i+3]) &&
+			  isdigit(bufptr[i+4]) && isspace(bufptr[i+5])) {
+		    /* probably the HOUR:MINUTE */
+		    for (i+=5;isspace(bufptr[i]); i++);
+	       }
+	  } else if ((bufptr[i+1] == ':') && isdigit(bufptr[i+2]) &&
+		     isdigit(bufptr[i+3])) {
 	       /* probably the HOUR:MINUTE */
-	       for (i+=5;isspace(bufptr[i]); i++);
+	       for (i+=4;isspace(bufptr[i]); i++);
 	  }
-     }
-     
+	  
+     }     
      /* Point at supposed start-of-fileIntName */
      dirname = alias = &bufptr[i]; 
      
-     if (dirname[strlen(dirname)-1] == '/')
-	  dirname[strlen(dirname)-1] = '\0';
      
      /* Skip . and .. */
      if (!strcmp(dirname, ".") || !strcmp(dirname, ".."))
@@ -1400,8 +1434,13 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 	       GSsetType(gs, A_DIRECTORY);
 	       sprintf (tmpstr, "%s [%s]", IntName, datestr);
 	       GSsetTitle(gs, tmpstr);
-	       sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
-			GSgetPath(gs), IntName);
+
+	       if (strcmp(ftpuser, "anonymous") != 0)
+ 		    sprintf(tmpstr, "%s%s/", URLget(ftpurl), IntName);
+	       else
+		    sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
+			     GSgetPath(gs), IntName);
+
 	       GSsetPath(gs, tmpstr);
 	       if (GFTPplus) {
 		    GSsetModDate(gs, datestr);
@@ -1419,8 +1458,12 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 	       GSsetType(gs, theName[0]);
 	       sprintf (tmpstr, "%s (%s) [%s]", IntName, sizestr, datestr);
 	       GSsetTitle(gs, tmpstr);
-	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-		       GSgetPath(gs), theName+1);
+	       if (strcmp(ftpuser, "anonymous") != 0)
+ 		    sprintf(tmpstr, "%s%s",URLget(ftpurl), theName+1);
+	       else
+		    sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
+			    GSgetPath(gs), theName+1);
+
 	       GSsetPath(gs, tmpstr);
 	       if (GFTPplus) {
 		    GSsetModDate(gs, datestr);
@@ -1431,6 +1474,10 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 	  break;
 	  
      case 'd': /* Now treat as regular directory */
+ 	  if (dirname[strlen(dirname)-1] == '/')
+	       dirname[strlen(dirname)-1] = '\0';
+      
+
 	  /* Display name in theName */
 	  sprintf(theName, "%c%s", A_DIRECTORY, dirname);
 
@@ -1449,8 +1496,11 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 			 *cp1 = '*';
 	       }
 	  
-	  sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs), GSgetPath(gs), 
-		   IntName);
+	  if (strcmp(ftpuser, "anonymous") != 0) 
+ 	       sprintf (tmpstr, "%s%s/", URLget(ftpurl), IntName);
+	  else
+	       sprintf (tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs), GSgetPath(gs), 
+			IntName);
 	  GSsetPath(gs, tmpstr);
 	  if (GFTPplus) {
 		  GSsetModDate(gs, datestr);
@@ -1468,7 +1518,12 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 	  GSsetType(gs, theName[0]);
 	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
-	  sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs), GSgetPath(gs), theName+1);
+	  if (strcmp(ftpuser, "anonymous") != 0)
+ 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+	  else
+	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
+		       GSgetPath(gs), theName+1);
+
 	  GSsetPath(gs, tmpstr);
 	  if (GFTPplus) {
 		  GSsetModDate(gs, datestr);
@@ -1482,12 +1537,8 @@ ParseUnixList(ftp, bufptr, IntName, theName, cols, gs)
 static char vmsprevline[64];
 
 int
-ParseVMSList(ftp, bufptr, IntName, theName, gs)
-  FTP  *ftp;
-  char *bufptr;
-  char *IntName;
-  char *theName;
-  GopherObj *gs;
+ParseVMSList(FTP  *ftp, char *bufptr, char *IntName, char *theName,
+	     GopherObj *gs, char *ftpuser, char *ftppass)
 {
      int i, j;
      int objsize;
@@ -1541,8 +1592,12 @@ ParseVMSList(ftp, bufptr, IntName, theName, gs)
      }
      switch (i = GSgetType(gs)) {
      case A_DIRECTORY:
-	  sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
-		  GSgetPath(gs), theName+1);
+	  if (strcmp(ftpuser, "anonymous") != 0)
+ 	       sprintf(tmpstr, "%s%s/", URLget(ftpurl), theName+1);
+	  else
+	       sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
+		       GSgetPath(gs), theName+1);
+
 	  GSsetPath(gs, tmpstr);
 	  sprintf (tmpstr, "%s [%s]", theName+1, datestr);
 	  GSsetTitle(gs, tmpstr);
@@ -1550,8 +1605,11 @@ ParseVMSList(ftp, bufptr, IntName, theName, gs)
 	  break;
      default: 
 	  
-	  sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-		  GSgetPath(gs), theName+1);
+	  if (strcmp(ftpuser, "anonymous") != 0)
+ 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+	  else
+	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
+		       GSgetPath(gs), theName+1);
 	  GSsetPath(gs, tmpstr);
 	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
@@ -1563,12 +1621,8 @@ ParseVMSList(ftp, bufptr, IntName, theName, gs)
 
 
 int
-ParseOS2List(ftp, bufptr, IntName, theName, gs)
-  FTP  *ftp;
-  char *bufptr;
-  char *IntName;
-  char *theName;
-  GopherObj *gs;
+ParseOS2List(FTP *ftp, char *bufptr, char *IntName, char *theName,
+	     GopherObj *gs, char *ftpuser, char *ftppass)
 {
      int i;
      int objsize;
@@ -1625,17 +1679,25 @@ ParseOS2List(ftp, bufptr, IntName, theName, gs)
 
      switch (i = GSgetType(gs)) {
      case A_DIRECTORY:
-	  sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
-		  GSgetPath(gs), theName+1);
+	  if (strcmp(ftpuser, "anonymous") != 0)
+ 	       sprintf(tmpstr, "%s%s/", URLget(ftpurl), theName+1);
+	  else
+	       sprintf(tmpstr, "ftp:%s@%s/%s/", GSgetHost(gs),
+		       GSgetPath(gs), theName+1);
+
 	  GSsetPath(gs, tmpstr);
 	  sprintf (tmpstr, "%s [%s]", theName+1, datestr);
 	  GSsetTitle(gs, tmpstr);
 	  return(A_DIRECTORY);
 	  break;
+
      default: 
 	  i = GSsetType(gs, theName[0]);
-	  sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
-		  GSgetPath(gs), theName+1);
+	  if (strcmp(ftpuser, "anonymous") != 0)
+ 	       sprintf(tmpstr, "%s%s", URLget(ftpurl), theName+1);
+	  else
+	       sprintf(tmpstr, "ftp:%s@%s/%s", GSgetHost(gs),
+		       GSgetPath(gs), theName+1);
 	  GSsetPath(gs, tmpstr);
 	  sprintf (tmpstr, "%s (%s) [%s]", theName+1, sizestr, datestr);
 	  GSsetTitle(gs, tmpstr);
@@ -1647,12 +1709,8 @@ ParseOS2List(ftp, bufptr, IntName, theName, gs)
 
 
 
-
 int
-GopherType(ftp, bufptr, theName)
-  FTP  *ftp;
-  char *bufptr;
-  char *theName;
+GopherType(FTP  *ftp, char *bufptr, char *theName)
 {
      int last;
      
@@ -1681,14 +1739,11 @@ GopherType(ftp, bufptr, theName)
 /* At this point we're looking at a file */
 
 int
-GopherFile(ftp, buf, theName)
-  FTP  *ftp;
-  char *buf;
-  char *theName;
+GopherFile(FTP  *ftp, char *buf, char *theName)
 {
      char	Gtype;
      int	last;
-     char	tmpName[SLEN];	
+     char	tmpName[256];	
      Extobj     *ext;
      char *slashp, **textn;
      
@@ -1705,14 +1760,22 @@ GopherFile(ftp, buf, theName)
 	  tmpName[last] = '\0';
      }
      
+     ext = EXnew();
      /* At this point we're looking at a file */
      if (GDCViewExtension(Config, buf, &ext)) {  
 	  Gtype = EXgetObjtype(ext);
 	  
 	  sprintf(theName, "%c%s", Gtype, tmpName);
+	  EXdestroy(ext);
 	  return(Gtype);
      }
 
+     if (EXAcasedSearch(Config->Extensions, ext, buf, EXT_DECODER)) {
+	  EXdestroy(ext);
+	  sprintf(theName, "%c%s", A_UNIXBIN, tmpName);
+	  return(A_UNIXBIN);
+     }
+
      /** Hack for some notable looking text files **/
      
      if ((slashp = strrchr(buf, '/')) == NULL)
@@ -1729,12 +1792,11 @@ GopherFile(ftp, buf, theName)
 }
 
 #else
+#include "command.h"
 
-GopherFTPgw(sockfd, Selstr)
-  int sockfd;
-  char *Selstr;
+GopherFTPgw(int sockfd, char *ftpstr, CMDobj *cmd)
 {
-     Abortoutput(sockfd, "Sorry, this server does not have ftp capabilities");
+     Die(sockfd, 501, "Sorry, this server does not have ftp capabilities");
 }
 
    
