@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.30
- * 1994/12/08 05:23:00
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopherd/gopherdconf.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.44 $
+ * $Date: 1995/09/26 04:59:46 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopherd/gopherdconf.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,49 @@
  * Routines to parse the gopherd.conf file.
  *********************************************************************
  * Revision History:
- * gopherdconf.c,v
+ * $Log: gopherdconf.c,v $
+ * Revision 3.44  1995/09/26  04:59:46  lindner
+ * Fix abort routines, change to Die/Warn hash over access routines..
+ *
+ * Revision 3.43  1995/09/25  22:10:36  lindner
+ * blobbo..
+ *
+ * Revision 3.42  1995/09/25  05:02:36  lindner
+ * Convert to ANSI C
+ *
+ * Revision 3.41  1995/09/09  03:20:27  lindner
+ * Better Error checks
+ *
+ * Revision 3.40  1995/08/30  20:13:35  lindner
+ * Remove tixfile
+ *
+ * Revision 3.39  1995/06/30  20:33:30  lindner
+ * Add group authentication
+ *
+ * Revision 3.38  1995/06/02  17:24:38  lindner
+ * Add the ability to specify alternate password files for unixfile auth.
+ *
+ * Revision 3.37  1995/04/15  07:14:18  lindner
+ * Add http port to gopherd.conf
+ *
+ * Revision 3.36  1995/02/16  22:32:39  lindner
+ * HTML icon support
+ *
+ * Revision 3.35  1995/02/13  19:07:06  lindner
+ * Add MaxConnection limit for total server
+ *
+ * Revision 3.34  1995/02/11  06:21:26  lindner
+ * remove pid junk
+ *
+ * Revision 3.33  1995/02/06  22:14:50  lindner
+ * remove unused variables
+ *
+ * Revision 3.32  1995/02/02  17:13:54  lindner
+ * Fix memory leaks
+ *
+ * Revision 3.31  1994/12/15  17:33:19  lindner
+ * Add BlockScript: keyword support
+ *
  * Revision 3.30  1994/12/08  05:23:00  lindner
  * Fix wording problem in integrity check
  *
@@ -137,12 +179,16 @@ GDCnew()
 
      gdc->Extensions   = EXAnew();
 
+#ifndef NO_AUTHENTICATION
      gdc->Sites        = SiteArrayNew();
      gdc->Securityon   = FALSE;
 
-     gdc->Authroutines = AUTHAnew(5);
-     gdc->Authitems    = AUTHITEMSnew(5);
+     gdc->Authroutines = AUTHAnew(3);
+     gdc->Authitems    = AUTHITEMSnew(3);
      gdc->Serverpw     = STRnew();
+     gdc->groupdir     = STRnew();
+     gdc->Defaccess    = ACC_FULL;
+#endif
 
      gdc->RunFromInetd = FALSE;
      gdc->Caching      = TRUE;
@@ -151,11 +197,9 @@ GDCnew()
      gdc->Logfile      = STRnew();
      gdc->Errorfile    = STRnew();
      gdc->Data_Dir     = STRnew();
-     gdc->Pid_dir      = STRnew();
      gdc->Hostname     = STRnew();
      gdc->Port         = 0;
      gdc->chroot       = TRUE;
-     gdc->Defaccess    = ACC_FULL;
      gdc->BummerMsg    = STRnew();
      
      gdc->Admin        = STRnew();
@@ -170,37 +214,40 @@ GDCnew()
      gdc->TZ           = 0;
      gdc->VeronicaIndex = TRUE;
 
-     gdc->Tixfile      = STRnew();
-
      STRset(gdc->Logfile, "");
      STRset(gdc->Errorfile, "");
      STRset(gdc->BummerMsg, "");
      STRset(gdc->Hostname, "");
 
-     gdc->other_dirs = STAnew(10);
-     gdc->other_gdcs = STAnew(10);
+     gdc->other_dirs = STAnew(2);
+     gdc->other_gdcs = STAnew(2);
 
-     gdc->FileSeparators = STAnew(10);
+     gdc->FileSeparators = STAnew(5);
 
+     gdc->BlkScriptBlocks = STAnew(5);
+     gdc->BlkScripts      = STAnew(5);
+
+     gdc->MaxConnections  = -1;
+     gdc->HTTPport        = 80;
      return(gdc);
 }
 
 void
-GDCdestroy(gdc)
-  GDCobj *gdc;
+GDCdestroy(GDCobj *gdc)
 {
+#ifndef NO_AUTHENTICATION
      EXAdestroy(gdc->Extensions);
      SiteArrDestroy(gdc->Sites);
      AUTHAdestroy(gdc->Authroutines);
      AUTHITEMSdestroy(gdc->Authitems);
      STRdestroy(gdc->Serverpw);
+     STRdestroy(gdc->groupdir);
+#endif
 
      STRdestroy(gdc->Logfile);
      STRdestroy(gdc->BummerMsg);
      STRdestroy(gdc->Hostname);
      STRdestroy(gdc->Lang);
-     STRdestroy(gdc->Tixfile);
-     STRdestroy(gdc->Pid_dir);
      STRdestroy(gdc->Abstract);
 
      /** etc..**/
@@ -208,6 +255,9 @@ GDCdestroy(gdc)
      STAdestroy(gdc->other_dirs);
      STAdestroy(gdc->other_gdcs);
      STAdestroy(gdc->FileSeparators);
+
+     STAdestroy(gdc->BlkScriptBlocks);
+     STAdestroy(gdc->BlkScripts);
      free(gdc);
 }
 
@@ -216,8 +266,7 @@ static FILE *GDCfiles[MAXGDCFLEVEL]; /* can 'include' up to ten levels deep */
 static int  GDCfilenum = -1;
 
 static void
-GDCfilePush(f)
-  FILE *f;
+GDCfilePush(FILE *f)
 {
      if ((++GDCfilenum) < MAXGDCFLEVEL)
 	  GDCfiles[GDCfilenum] = f;
@@ -229,9 +278,7 @@ GDCfilePush(f)
 
 static
 char *
-GDCfileGetline(inputline, size)
-  char *inputline;
-  int  size;
+GDCfileGetline(char *inputline, int  size)
 {
      char* farg;
 
@@ -256,14 +303,15 @@ GDCfileGetline(inputline, size)
 
 static
 boolean
-GDCtokens(gdc, token, rest)
-  GDCobj *gdc;
-  char *token;
-  char *rest;
+GDCtokens(GDCobj *gdc, char *token, char *rest)
 {
      boolean success = TRUE;
 
-     if (strcasecmp(token, "ACCESS") == 0) {
+
+     if (strcasecmp(token, "ADMIN")==0)
+	  GDCsetAdmin(gdc, rest);
+#ifndef NO_AUTHENTICATION
+     else if (strcasecmp(token, "ACCESS") == 0) {
 	  Accesslevel moo;
 
 	  success = SiteProcessLine(gdc->Sites, rest, gdc->Defaccess);
@@ -274,9 +322,20 @@ GDCtokens(gdc, token, rest)
 
 	  gdc->Securityon = TRUE;
      }
-
-     else if (strcasecmp(token, "ADMIN")==0)
-	  GDCsetAdmin(gdc, rest);
+     else if (strcasecmp(token, "AUTHSCRIPT")==0) {
+	  success = AUTHAprocessLine(gdc->Authroutines, rest);
+     }
+     else if (strcasecmp(token, "AUTHITEM")==0) {
+	  success = AUTHITEMSprocessLine(gdc->Authitems, rest);
+     }
+     else if (strcasecmp(token, "SERVERPW")==0) {
+	  GDCsetPW(gdc, rest);
+     } 
+     else if (strcasecmp(token, "GROUPDIR")==0) {
+	  GDCsetGroupdir(gdc, rest);
+	  success = GGTfromDir(rest);
+     }
+#endif
      else if (strcasecmp(token, "ADMINEMAIL")==0)
 	  GDCsetAdminEmail(gdc, rest);
      else if (strcasecmp(token, "HOSTALIAS")==0)
@@ -291,8 +350,6 @@ GDCtokens(gdc, token, rest)
 	  GDCsetLogfile(gdc, rest);
      else if (strcasecmp(token, "ERRORFILE")==0)
 	  GDCsetErrorfile(gdc, rest);
-     else if (strcasecmp(token, "PIDS_DIRECTORY")==0)
-	  GDCsetPiddir(gdc, rest);
      else if (strcasecmp(token, "GEOG")==0)
 	  GDCsetGeog(gdc, rest);
      else if (strcasecmp(token, "BUMMERMSG")==0)
@@ -333,24 +390,17 @@ GDCtokens(gdc, token, rest)
      else if (strcasecmp(token, "CACHETIME")==0) {
 	  GDCsetCachetime(gdc, atoi(rest));
      }
-     else if (strcasecmp(token, "AUTHSCRIPT")==0) {
-	  success = AUTHAprocessLine(gdc->Authroutines, rest);
-     }
-     else if (strcasecmp(token, "AUTHITEM")==0) {
-	  success = AUTHITEMSprocessLine(gdc->Authitems, rest);
-     }
-     else if (strcasecmp(token, "SERVERPW")==0) {
-	  GDCsetPW(gdc, rest);
-     } 
      else if (strcasecmp(token, "FILESEP")==0) {
 	  GDCaddFileSep(gdc, rest);
-     } 
+     } else if (strcasecmp(token, "HTTPPORT") == 0) {
+	  GDCsetHTTPport(gdc, atoi(rest));
+     }
      else if (strcasecmp(token, "INCLUDE") == 0) {
 	  /** Check to see if we can open the file.. **/
 	  /** Then push it onto the stack.. **/
 	  FILE *gdcfile;
 
-	  if (*rest != '/') {
+	  if (*rest != '/' && *rest != '.') {
 	       char tmpstr[256];
 
 	       sprintf(tmpstr, "%s/%s", GDCDIR, rest);
@@ -363,6 +413,22 @@ GDCtokens(gdc, token, rest)
 	  GDCfilePush(gdcfile);
 	  
      }
+     else if (strcasecmp(token, "BLOCKSCRIPT") == 0) {
+	  char *cp;
+
+	  /** Block name is first argument, script to run is second **/
+
+	  cp = strchr(rest, ' ');
+	  if (cp != NULL) {
+	       *cp = '\0';
+	       cp++;
+	       
+	       GDCpushBlockScript(gdc, rest, cp);
+
+	  } else {
+	       success = FALSE;
+	  }
+     }
      else if (strcasecmp(token, "AUXCONF")==0) {
 	  /** Directory is first arg, conf file is second **/
 	  char *cp;
@@ -386,6 +452,12 @@ GDCtokens(gdc, token, rest)
 	       GDCpushOtherGDC(gdc, rest, cp+1);
 	  }
      }
+     else if (strcasecmp(token, "MAXCONNECTIONS")==0) {
+	  int numconns = atoi(rest);
+
+	  if (numconns > 1)
+	       GDCsetMaxconns(gdc, atoi(rest));
+     }
      else
 	  success = FALSE;
      
@@ -394,9 +466,7 @@ GDCtokens(gdc, token, rest)
 
 
 void
-GDCfromFile(gdc, filename)
-  GDCobj *gdc;
-  char *filename;
+GDCfromFile(GDCobj *gdc, char *filename)
 {
      FILE *gdcfile;
      char inputline[512];
@@ -417,7 +487,7 @@ GDCfromFile(gdc, filename)
 
 	  inputlen = strlen(inputline);	  
 
-	  while (inputline[inputlen-1] == '\\') {
+	  while ((inputlen > 1) && inputline[inputlen-1] == '\\') {
 	       inputline[inputlen-1] = '\n';
 	       if (!GDCfileGetline(inputline + inputlen, 
 			      sizeof(inputline)-inputlen))
@@ -457,10 +527,7 @@ GDCfromFile(gdc, filename)
 
 
 void
-GDCpushOtherGDC(gdc, dir, conffile)
-  GDCobj *gdc;
-  char *dir;
-  char *conffile;
+GDCpushOtherGDC(GDCobj *gdc, char *dir, char *conffile)
 {
      String *temp;
 
@@ -475,15 +542,26 @@ GDCpushOtherGDC(gdc, dir, conffile)
 }
 
 
+void
+GDCpushBlockScript(GDCobj *gdc, char *blockname, char *scriptname)
+{
+     String *temp;
 
+     temp = STRnew();
+     STRset(temp, blockname);
+     STApush(gdc->BlkScriptBlocks, temp);
+
+     STRset(temp, scriptname);
+     STApush(gdc->BlkScripts, temp);
+
+     STRdestroy(temp);
+}
 /*
  * Search for otherdir gdcs
  */
 
 char *
-GDCfindOtherGDC(gdc, dir)
-  GDCobj *gdc;
-  char *dir;
+GDCfindOtherGDC(GDCobj *gdc, char *dir)
 {
      int  i;
 
@@ -497,9 +575,7 @@ GDCfindOtherGDC(gdc, dir)
 
 
 boolean
-GDCevalDir(gdc, dir)
-  GDCobj *gdc;
-  char   *dir;
+GDCevalDir(GDCobj *gdc,  char *dir)
 {
      int  i, gdcdirlen;
      char *gdcdir, *conf;
@@ -513,7 +589,6 @@ GDCevalDir(gdc, dir)
 	  gdcdir = STAgetText(gdc->other_dirs, i);
 	  gdcdirlen = strlen(gdcdir);
 
-/*	  if (gdcdirlen <= dirlen && strncmp(dir, gdcdir, gdcdirlen)==0) {*/
 	  if (re_comp(gdcdir))
 	       return(TRUE);
 	  if (re_exec(dir)) {
@@ -528,92 +603,26 @@ GDCevalDir(gdc, dir)
 
 #ifndef NO_AUTHENTICATION
 AccessResult
-GDCCanSearch(gdc, hostname, ipnum, sessions)
-  GDCobj *gdc;
-  char   *hostname, *ipnum;
-  int     sessions;
+GDCaccess(GDCobj *gdc, char *hostname, char *ipnum, int sessions, 
+	  Accesslevel testfor)
 {
      AccessResult test;
 
      Debug("Testing %s/", hostname);
-     Debug("%s for searching\n", ipnum);
+     Debug("%s for access\n", ipnum);
 
      if (gdc->Securityon == FALSE)
 	  return(SITE_OK);
 
-     test = SiteAccess(gdc->Sites, hostname, ipnum, ACC_SEARCH, sessions);
+     test = SiteAccess(gdc->Sites, hostname, ipnum, testfor, sessions);
 
      return(test);
+     ;
 }
 
 
-
-AccessResult
-GDCCanRead(gdc, hostname, ipnum, sessions)
-  GDCobj *gdc;
-  char   *hostname, *ipnum;
-  int     sessions;
-{
-     AccessResult test;
-
-     Debug("Testing %s/", hostname);
-     Debug("%s for reading\n", ipnum);
-
-     if (gdc->Securityon == FALSE)
-	  return(SITE_OK);
-
-     test = SiteAccess(gdc->Sites, hostname, ipnum, ACC_READ, sessions);
-
-     return(test);
-}
-
-
-AccessResult
-GDCCanBrowse(gdc, hostname, ipnum, sessions)
-  GDCobj *gdc;
-  char   *hostname, *ipnum;
-  int    sessions;
-{
-     AccessResult test;
-
-     Debug("Testing %s/", hostname);
-     Debug("%s for browsing\n", ipnum);
-
-     if (gdc->Securityon == FALSE)
-	  return(SITE_OK);
-
-
-     test = SiteAccess(gdc->Sites, hostname, ipnum, ACC_BROWSE, sessions);
-
-     return(test);
-
-}
-
-
-AccessResult
-GDCCanFTP(gdc, hostname, ipnum, sessions)
-  GDCobj *gdc;
-  char   *hostname, *ipnum;
-  int     sessions;
-{
-     AccessResult test;
-
-     Debug("Testing %s/", hostname);
-     Debug("%s for ftping\n", ipnum);
-
-     if (gdc->Securityon == FALSE)
-	  return(SITE_OK);
-
-     test = SiteAccess(gdc->Sites, hostname, ipnum, ACC_FTP, sessions);
-
-     return(test);
-
-}
-
 char *
-GDCauthType(gdc, item)
-  GDCobj *gdc;
-  char         *item;
+GDCauthType(GDCobj *gdc, char *item)
 {
      if (item == NULL)
 	  return(NULL);
@@ -621,17 +630,24 @@ GDCauthType(gdc, item)
      return(AUTHITEMSfindType(gdc->Authitems, item));
 }
 
+char *
+GDCauthExtraArgs(GDCobj *gdc, char *item)
+{
+     if (item == NULL)
+	  return(NULL);
+
+     return(AUTHITEMSfindExtraArgs(gdc->Authitems, item));
+}
+
 AUTHresult
-GDCvalidate(gdc, type, username, password, hostname, hostip)
-  GDCobj *gdc;
-  char *type;
-  char *username, *password, *hostname, *hostip;
+GDCvalidate(GDCobj *gdc, char *type, char *username, 
+	    char *password, char *hostname, char *hostip, char *extra_args)
 {
      if (type == NULL)
 	  return(AUTHRES_OK);	/* If no auth method then it's okay. */
 
      return(AUTHAvalidate(gdc->Authroutines,type, username, password,
-			  hostname, hostip));
+			  hostname, hostip, extra_args));
 }
 
 /*
@@ -639,9 +655,7 @@ GDCvalidate(gdc, type, username, password, hostname, hostip)
  */
 
 char **
-GDCauthAsk(gdc, path)
-  GDCobj *gdc;
-  char   *path;
+GDCauthAsk(GDCobj *gdc, char *path)
 {
      AUTH *auth;
 
@@ -660,9 +674,7 @@ GDCauthAsk(gdc, path)
 
 /** Is this file ignored? **/
 boolean
-GDCignore(gdc, fname)
-  GDCobj *gdc;
-  char *fname;
+GDCignore(GDCobj *gdc, char *fname)
 {
      Extobj *ext;
 
@@ -679,10 +691,7 @@ GDCignore(gdc, fname)
 
 
 boolean 
-GDCViewExtension(gdc, fext, extin)
-  GDCobj *gdc;
-  char *fext;
-  Extobj **extin;
+GDCViewExtension(GDCobj *gdc, char *fext, Extobj **extin)
 {
      Extobj *ext;
      
@@ -700,10 +709,7 @@ GDCViewExtension(gdc, fext, extin)
 }
 
 boolean 
-GDCBlockExtension(gdc, fext, ext)
-  GDCobj *gdc;
-  char *fext;
-  Extobj *ext;
+GDCBlockExtension(GDCobj *gdc, char *fext, Extobj *ext)
 {
 
      if (EXAsearch(gdc->Extensions, ext, fext, EXT_BLOCK)) {
@@ -716,9 +722,7 @@ GDCBlockExtension(gdc, fext, ext)
 
 
 void
-GDCaddFileSep(gdc, expr)
-  GDCobj *gdc;
-  char   *expr;
+GDCaddFileSep(GDCobj *gdc, char *expr)
 {
      String *temp;
 
@@ -732,8 +736,7 @@ GDCaddFileSep(gdc, expr)
 }
 
 static boolean
-GDCisBlank(string)
-  char *string;
+GDCisBlank(char *string)
 {
      if (string == NULL || 
 	 strncmp(string, "blank", 5) == 0)
@@ -743,8 +746,7 @@ GDCisBlank(string)
 }
 
 void
-GDCintegrityCheck(gdc)
-  GDCobj *gdc;
+GDCintegrityCheck(GDCobj *gdc)
 {
      if (GDCisBlank(GDCgetSite(gdc)))
 	  fprintf(stderr, "** Please set the Site: line in your configuration file!\n");
