@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.32
- * 1994/12/06 20:56:41
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopherd/serverutil.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.52 $
+ * $Date: 1996/01/04 18:30:14 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopherd/serverutil.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,67 @@
  * utilities for the server.
  *********************************************************************
  * Revision History:
- * serverutil.c,v
+ * $Log: serverutil.c,v $
+ * Revision 3.52  1996/01/04  18:30:14  lindner
+ * Fix for Ustat on Linux, autoconf changes
+ *
+ * Revision 3.51  1995/11/06  20:53:40  lindner
+ * aix, no setproctitle
+ *
+ * Revision 3.50  1995/11/03  18:02:10  lindner
+ * Small null check
+ *
+ * Revision 3.49  1995/10/04  21:04:55  lindner
+ * NO_AUTH mod
+ *
+ * Revision 3.48  1995/09/28  22:58:53  lindner
+ * Minor fixes
+ *
+ * Revision 3.47  1995/09/28  21:11:31  lindner
+ * More fixes...
+ *
+ * Revision 3.46  1995/09/26  05:12:45  lindner
+ * Add include...
+ *
+ * Revision 3.45  1995/09/26  04:59:48  lindner
+ * Fix abort routines, change to Die/Warn hash over access routines..
+ *
+ * Revision 3.44  1995/09/25  22:09:51  lindner
+ * Remove static buf..
+ *
+ * Revision 3.43  1995/09/25  05:02:38  lindner
+ * Convert to ANSI C
+ *
+ * Revision 3.42  1995/05/01  05:42:15  lindner
+ * compatibility fixes
+ *
+ * Revision 3.41  1995/05/01  04:00:47  lindner
+ * Fixes for mindexd, etc from Steve Hsieh
+ *
+ * Revision 3.40  1995/02/25  20:49:05  lindner
+ * More timeouts..
+ *
+ * Revision 3.39  1995/02/11  06:22:49  lindner
+ * Fix for setvbuf parameters
+ *
+ * Revision 3.38  1995/02/07  19:34:08  lindner
+ * A little more readable
+ *
+ * Revision 3.37  1995/02/07  08:37:37  lindner
+ * Rewrite of mailfile/multifile parsing
+ *
+ * Revision 3.36  1995/02/07  07:03:45  lindner
+ * performance fixes
+ *
+ * Revision 3.35  1995/02/02  17:13:54  lindner
+ * Fix memory leaks
+ *
+ * Revision 3.34  1994/12/31  07:46:45  lindner
+ * Make shell script output line buffered for slow scripts
+ *
+ * Revision 3.33  1994/12/20  17:20:56  lindner
+ * Put environment variable setter here..
+ *
  * Revision 3.32  1994/12/06  20:56:41  lindner
  * Fix for multi-line bummer messages
  *
@@ -126,19 +186,16 @@
 #include "serverutil.h"
 #include "Debug.h"
 #include "Dirent.h"		/* For S_ISADIR */
+#include "gopherdconf.h"	/* For GDCaccess */
 
-#ifndef NOSYSLOG
+#if defined(HAVE_SYSLOG_H)
 #include <syslog.h>
 #else
 #define syslog(a,b)	fprintf(stderr, "%s\n", (b))
 #define openlog(a,b,c)	fprintf(stderr, "%s\n", (a))
 #endif
 
-#ifdef __STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
+#include <stdarg.h>		/* For ellipsis routines ... */
 
 /* Try using flock() if fcntl() won't let us lock. */
 #if !defined(USE_FLOCK) && !defined(F_SETLKW)
@@ -153,15 +210,7 @@
  * This finds the current peer and the time and  jams it into the
  * logfile (if any) and adds the message at the end
  */
-#ifdef __STDC__
 void LOGGopher(int sockfd, const char *fmt, ...)
-#else /* !__STDC__ */
-void
-LOGGopher(sockfd, fmt, va_alist)
-  int sockfd;
-  char *fmt;
-va_dcl
-#endif /* __STDC__ */
 {
      va_list args;
      char message[512];
@@ -174,11 +223,7 @@ va_dcl
      struct flock    lock;
 #endif
 
-#ifdef __STDC__
      va_start(args, fmt);
-#else
-     va_start(args);
-#endif
      
      (void) vsprintf(message, fmt, args);
 
@@ -243,11 +288,7 @@ va_dcl
  */
 
 void
-GplusError(sockfd, errclass, text, moretext)
-  int sockfd;
-  int errclass;
-  char *text;
-  char **moretext;
+GplusError(int sockfd, int errclass, char *text, char **moretext)
 {
      char outputline[256];
      int i;
@@ -279,14 +320,16 @@ GplusError(sockfd, errclass, text, moretext)
      if (!IsGplus)
 	  sprintf(outputline, "0%s\t\terror.host\t1\r\n", text);
      else
-	  sprintf(outputline, "--1\r\n%d %s <%s>\r\n0%s\r\n", 
+	  sprintf(outputline, "--1\r\n%d %s <%s>\r\n%s\r\n", 
 	     errclass, GDCgetAdmin(Config),
 	     GDCgetAdminEmail(Config), text);
 
+     (void) alarm(WRITETIMEOUT);
      if (writestring(sockfd, outputline)<0) {
 	  LOGGopher(sockfd, "Client went away!");
 	  exit(-1);
      }
+     (void) alarm(0);
 
      if (moretext != NULL)
 	  for (i=0; moretext[i] != NULL; i++) {
@@ -294,11 +337,16 @@ GplusError(sockfd, errclass, text, moretext)
 		    sprintf(outputline, "0%s\t\terror.host\t1\r\n", 
 			    moretext[i]);
 		    writestring(sockfd, outputline);
-	       } else 
+	       } else  {
+		    (void) alarm(WRITETIMEOUT);
 		    writestring(sockfd, moretext[i]);
+		    writestring(sockfd, "\n");
+		    (void) alarm(0);
+	       }
 	  }
-
+     (void) alarm(WRITETIMEOUT);
      writestring(sockfd, ".\r\n");
+     (void) alarm(0);
 
      close(sockfd);
 
@@ -312,21 +360,44 @@ GplusError(sockfd, errclass, text, moretext)
  */
 
 void
-Abortoutput(sockfd, errmsg)
-  int sockfd;
-  char *errmsg;
+Abortoutput(int sockfd, char *errmsg)
 {
      GplusError(sockfd, 1, errmsg, NULL);
 
 }
 
+void
+Die(int sockfd, int errorlevel, char *fmt, ...)
+{
+     va_list args;
+     char message[512];
+     
+     va_start(args, fmt);
+
+     vsprintf(message, fmt, args);
+     GplusError(sockfd, 1, message, NULL);
+     gopherd_exit(-1);
+}
+
+void
+Warn(int sockfd, int errorlevel, char *fmt, ...)
+{
+     va_list args;
+     char message[512];
+     
+     va_start(args, fmt);
+
+     vsprintf(message, fmt, args);
+     GplusError(sockfd, 1, message, NULL);
+}
+
+
 /*
  * only works if non-chroot really...
  */
 
 boolean
-Setuid_username(username)
-  char *username;
+Setuid_username(char *username)
 {
      struct passwd *pw;
 
@@ -388,8 +459,7 @@ static char legal_day[]         = "SunMonTueWedThuFriSat";
 static char legal_month[]       = "JanFebMarAprMayJunJulAugSepOctNovDec";
 static int  legal_numbers[]     = { 1, 31, 0, 23, 0, 59, 0, 60, 1969, 2199 };
 
-int is_mail_from_line(line)
-  char *line;     /* Line of text to be checked */
+int is_mail_from_line(char *line)
 {
      char *fields[MAX_FIELDS];
      char *sender_tail;
@@ -409,7 +479,7 @@ int is_mail_from_line(line)
 	  if (n == 0) sender_tail = lp;
      }
      
-     if (n < 8) return -200-n;
+     if (n < 7) return -200-n;
      
      fp = fields;
      
@@ -436,16 +506,15 @@ int is_mail_from_line(line)
 }
 
 
-int
-is_multipartfile(line)
-  char *line;
+Splittype
+is_multipartfile(char *line)
 {
      int success, i;
      int numitems = GDCgetNumFileSep(Config);
 
      success = is_mail_from_line(line);
      if (success == 0)
-	  return(-1);
+	  return(SPLIT_MAIL);
 
      for (i=0; i< numitems; i++) {
 	  re_comp(GDCgetFileSep(Config, i));
@@ -453,19 +522,105 @@ is_multipartfile(line)
 	       return(i);
      }
      
-     return(-2);
+     return(SPLIT_UNKNOWN);
 }
 
 
+#define NO_SUBJECT "<no subject>"
+
+void
+process_mailfile(int sockfd, char *Mailfname)
+{
+     FILE       *Mailfile;
+     char       Zeline[MAXLINE];
+     char       outputline[MAXLINE];
+     char       Title[MAXLINE];
+     long       Startbyte=0, Endbyte=0, Bytecount=0;
+     boolean    foundtitle = 0;
+     char       *p;
+     Splittype  septype = SPLIT_UNKNOWN;
+
+     Debug("process_mailfile %s\n",Mailfname);
+
+     Mailfile = rfopen(Mailfname, "r");
+
+     if ((Mailfile == NULL) || (fgets(Zeline, MAXLINE, Mailfile) == NULL)) {
+	  Die(sockfd, 404, "Cannot access file '%s'", Mailfname);
+	  return;
+     }
+
+     septype = is_multipartfile(Zeline);
+     Bytecount += strlen(Zeline);
+
+     /** Read through the file, finding sections **/
+     while (fgets(Zeline, MAXLINE, Mailfile) != NULL) {
+	  if (!foundtitle) {
+	       if (septype==SPLIT_MAIL) {
+		    if (strncmp(Zeline,"Subject: ",9)==0) {
+			 foundtitle = TRUE;
+			 /* trim out the white space.. */
+			 p = Zeline + 8;
+			 while ((*p == ' ')||(*p == '\t'))
+			      p++;
+			 if (*p == '\n')
+			      strcpy(Title, NO_SUBJECT);
+			 else
+			      strcpy(Title,p);
+			 
+			 ZapCRLF(Title);
+			 Debug("Found title %s\n", Title);
+		    } else if (strcmp(Zeline, "\n")==0) {
+			 foundtitle = TRUE;
+			 strcpy(Title, NO_SUBJECT);
+			 Debugmsg("No subject found - using default\n");
+		    }
+	       } else {
+		    /** Not SPLIT_MAIL **/
+		    strcpy(Title, Zeline);
+		    ZapCRLF(Title);
+		    foundtitle = TRUE;
+	       }
+	  }
+	  else if (is_multipartfile(Zeline) == septype) {
+	       Endbyte = Bytecount;
+	       foundtitle = FALSE;
+
+	       if (Endbyte != 0) {
+		    sprintf(outputline, "0%s\tR%d-%d-%s\t%s\t%d\r\n", 
+			    Title, Startbyte, Bytecount, Mailfname,
+			    Zehostname, GopherPort);
+		    if (writestring(sockfd, outputline) < 0)
+			 gopherd_exit(-1);
+		    Startbyte=Bytecount;
+		    *Title = '\0';
+	       }
+	  }
+
+	  Bytecount += strlen(Zeline);
+     }
+
+     if (*Title != '\0') {
+	  sprintf(outputline, "0%s\tR%d-%d-%s\t%s\t%d\r\n", 
+		  Title, Startbyte, Bytecount, Mailfname, 
+		  Zehostname, GopherPort);
+	  if (writestring(sockfd, outputline)<0)
+	       gopherd_exit(-1);
+     }	  
+
+}
+
+
+
+
+
 /* 
  * Return the basename of a filename string, i.e. everything
  * after the last "/" character. 
  */
 
-char *mtm_basename(string)
-  char *string;
+char *mtm_basename(char *string)
 {
-     static   char *buff;
+     char *buff;
      buff = string + strlen(string); /* start at last char */
      while (*buff != '/' && buff > string)
 	  buff--;
@@ -482,10 +637,7 @@ char *mtm_basename(string)
  */
 
 boolean
-Cachetimedout(cache, secs, dir)
-  char *cache;
-  int secs;
-  char *dir;
+Cachetimedout(char *cache, int secs, char *dir)
 {
      STATSTR       buf;
      int           result;
@@ -515,8 +667,7 @@ Cachetimedout(cache, secs, dir)
  */
 
 boolean
-isadir(path)
-  char *path;
+isadir(char *path)
 {
      static STATSTR buf;
      int result;
@@ -544,30 +695,22 @@ isadir(path)
  * systems
  */
 
-#ifdef __STDC__
 void
 ServerSetArgv(const char *fmt, ...)
-#else /* !__STDC__ */
-void
-ServerSetArgv(fmt, va_alist)
-  char *fmt;
-va_dcl
-#endif /* __STDC__ */
 {
+#if defined(SETPROCTITLE) && !defined(__sgi) && !defined(_AUX_SOURCE) && !defined(sgi) && !defined(USG) && !defined(_AIX)
+
      va_list args;
+
      register char *p;
      register int i;
      char buf[MAXLINE];
 
-#if defined(SETPROCTITLE) && !defined(__sgi) && !defined(_AUX_SOURCE) && !defined(sgi) && !defined(USG)
+
 
      Argv[1] = NULL;
 
-# ifdef __STDC__
      va_start(args, fmt);
-# else /* !__STDC__ */
-     va_start(args);
-# endif /* __STDC__ */
      (void) vsprintf(buf, fmt, args);
      
      va_end(args);
@@ -591,14 +734,12 @@ va_dcl
 }
 
 FILE*
-Gpopen(sockfd, cmd, rw)
-  int  sockfd;
-  char *cmd;
-  char *rw;
+Gpopen(int sockfd, char *cmd, char *rw)
 {
      int inquote = 0;
      int insquote = 0;
      int i;
+     FILE *theproc;
 
      /** Strip out the naughty bits..  **/
      for (i=0; cmd[i] != '\0'; i++) {
@@ -649,5 +790,51 @@ Gpopen(sockfd, cmd, rw)
 	  }
      }
 
-     return(popen(cmd, rw));
+     theproc = popen(cmd, rw);
+     if (theproc != NULL)
+	  setvbuf(theproc, NULL, _IOLBF, 0);
+
+     return(theproc);
 }
+
+/*
+ * Set an environment variable 
+ */
+
+void
+SetEnvironmentVariable(char *variable, char *value)
+{
+     char *tmpputenv;
+     
+     if (variable == NULL)
+	  return;
+
+     if (value == NULL)
+	  value = "";
+
+     tmpputenv = (char*) malloc(strlen(variable) + strlen(value) + 2);
+
+     if (!tmpputenv)
+	  return;
+
+     sprintf(tmpputenv, "%s=%s", variable, value);
+     putenv(tmpputenv);
+}
+
+#ifndef NO_AUTHENTICATION
+void
+CheckAccess(int sockfd, Accesslevel testfor)
+{
+     AccessResult result;
+
+     result = GDCaccess(Config, CurrentPeerName, CurrentPeerIP,
+			     ActiveSessions, testfor);
+     
+     if (result == SITE_NOACCESS) {
+	  Die(sockfd, 411, GDCgetBummerMsg(Config));
+     } else if (result == SITE_TOOBUSY) {
+	  Die(sockfd, 503, "Sorry, too busy now...");
+     }
+}
+
+#endif
