@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.27
- * 1994/11/05 03:20:02
- * /home/mudhoney/GopherSrc/CVS/gopher+/gopherd/waisgopher.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.33 $
+ * $Date: 1995/10/05 03:29:11 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/gopherd/waisgopher.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,25 @@
  * Routines to translate from gopher protocol to wais protocol
  *********************************************************************
  * Revision History:
- * waisgopher.c,v
+ * $Log: waisgopher.c,v $
+ * Revision 3.33  1995/10/05  03:29:11  lindner
+ * Remove unused symbol
+ *
+ * Revision 3.32  1995/09/28  21:11:32  lindner
+ * More fixes...
+ *
+ * Revision 3.31  1995/09/26  04:59:49  lindner
+ * Fix abort routines, change to Die/Warn hash over access routines..
+ *
+ * Revision 3.30  1995/02/16  22:32:41  lindner
+ * HTML icon support
+ *
+ * Revision 3.29  1995/02/01  21:41:48  lindner
+ * Fix for FreeWAIS 0.4
+ *
+ * Revision 3.28  1995/01/04  17:36:31  lindner
+ * Fix for freewais-sf on linux
+ *
  * Revision 3.27  1994/11/05  03:20:02  lindner
  * Add logging, add period output to wais-gopher gateway
  *
@@ -143,12 +161,25 @@
 #define DOCIDSIZE 5120
 
 
+/*** Need for freewais-sf on some platforms ***/
+#ifndef MIN
+#define MIN(x,y) (((x) < (y)) ? (x) : (y))
+#endif
+
+
+
 /**** Needed by wais-8-b5... Ick... ***/
 
 char *sdir=NULL;
 char *cdir=NULL;
 
 
+/**** For FreeWAIS 0.4 ****/
+#ifdef FREEWAIS_0_4
+#  define init_connection   FW_init_connection
+#  define close_connection  FW_close_connection
+#endif
+
 /* encode a variable length integer - used by gopher_to_docid */
 static unsigned char* 
 integer_encode(cp,value) 
@@ -179,8 +210,7 @@ integer_encode(cp,value)
 
 /**** Used by something in the source reading routines. ***/
 void
-PrintStatus(str)
-  char * str;
+PrintStatus(char *str)
 {
      return;
 }
@@ -554,7 +584,7 @@ display_search_response(response, hostname, port, dbname, SourceName,
 	       }
 	       
 	       if (view == NULL)
-		    GDtoNet(gd, sockfd, GSFORM_G0, Gticket);
+		    GDtoNet(gd, sockfd, GSFORM_G0, Gticket, NULL);
 	       else {
 		    if (isgplus)
 			 GSsendHeader(sockfd, -1);
@@ -562,7 +592,7 @@ display_search_response(response, hostname, port, dbname, SourceName,
 		    if (strcasecmp(view, "application/gopher+-menu")==0)
 			 GDplustoNet(gd, sockfd, NULL, Gticket);
 		    else
-			 GDtoNet(gd, sockfd, GSFORM_G0, Gticket);
+			 GDtoNet(gd, sockfd, GSFORM_G0, Gticket, NULL);
 	       }
 
 	  }
@@ -593,12 +623,12 @@ MyReadSource(Moo, filename, sockfd)
      FILE *dotsrc;
      char  *cp;
      if (filename == NULL)
-	  Abortoutput(sockfd, "No Filename!");
+	  Die(sockfd, 404, "No Filename!");
 
      if (*filename == '(' && filename[strlen(filename)-1] == ')') {
 	  cp = strchr(filename, ' ');
 	  if (cp==NULL)
-	       Abortoutput(sockfd, "malformed filename");
+	       Die(sockfd, 404, "malformed filename");
 	  *cp = '\0';
 	  strcpy(Moo->server, filename+1);
 	  *cp = ' ';
@@ -606,7 +636,7 @@ MyReadSource(Moo, filename, sockfd)
 	  filename = cp+1;
 	  cp = strchr(filename, ' ');
 	  if (cp==NULL)
-	       Abortoutput(sockfd, "malformed filename");
+	       Die(sockfd, 404, "malformed filename");
 	  *cp = '\0';
 	  strcpy(Moo->database, filename);
 	  *cp = ' ';
@@ -617,11 +647,8 @@ MyReadSource(Moo, filename, sockfd)
      }
      else {
 	  dotsrc = rfopen(filename, "r");
-	  if (dotsrc == NULL) {
-	       char tmpstr[256];
-	       sprintf(tmpstr, "File '%s' does not exist", filename);
-	       Abortoutput(sockfd, tmpstr), gopherd_exit(-1);
-	  }
+	  if (dotsrc == NULL)
+	       Die(sockfd, 404, "File '%s' does not exist", filename);
 
 	  ReadSource(Moo, dotsrc);
 	  fclose(dotsrc);
@@ -689,21 +716,16 @@ void SearchRemoteWAIS(sockfd, SourceName, cmd, view)
 	  connection = NULL;
      
      else if ((connection=connect_to_server(server_name,atoi(service))) == NULL) 
-     {
-	  char Errorstr[256];
-
-	  sprintf(Errorstr,"Error openning connection to %s via service %s.", 
-		  server_name, service);
-	  Abortoutput(sockfd, Errorstr);
-	  gopherd_exit(-1);
-     }
+	  Die(sockfd, 504, "Error openning connection to %s via service %s.", 
+	      server_name, service);
      
      request_message = (char*)s_malloc((size_t)MAX_MESSAGE_LEN * sizeof(char));
      response_message = (char*)s_malloc((size_t)MAX_MESSAGE_LEN * sizeof(char));
      {
 	  char userInfo[256];
 
-	  sprintf(userInfo, "waisgopher %s, from host: %s", VERSION, Zehostname);
+	  sprintf(userInfo, "waisgopher, from host: %s", Zehostname);
+
 	  init_connection(request_message, response_message,
 			  MAX_MESSAGE_LEN,
 			  connection,
@@ -715,7 +737,7 @@ void SearchRemoteWAIS(sockfd, SourceName, cmd, view)
 	generate_search_apdu(request_message + HEADER_LENGTH, 
 			     &request_buffer_length, 
 			     keywords, database, NULL, WAISMAXHITS))
-	  Abortoutput(sockfd, "request too large");
+	  Die(sockfd, 504, "request too large");
      
      
      if(0 ==
@@ -730,11 +752,8 @@ void SearchRemoteWAIS(sockfd, SourceName, cmd, view)
 	       fclose(connection);
 	       if ((connection=connect_to_server(server_name,atoi(service))) == NULL) 
 	       {
-		    char Errorstr[256];
-
-		    sprintf(Errorstr, "Error openning connection to %s via service %s.\n",
-			   server_name, service);
-		    Abortoutput(sockfd, Errorstr);
+		    Die(sockfd, 504, "Error openning connection to %s via service %s.\n",
+			server_name, service);
 	       }
 
 	       if(0 ==
@@ -745,10 +764,10 @@ void SearchRemoteWAIS(sockfd, SourceName, cmd, view)
 				    connection,
 				    false /* true verbose */
 				    ))
-		    Abortoutput(sockfd, "really couldn't deliver message");
+		    Die(sockfd, 504, "really couldn't deliver message");
 	  }
 	  else
-	       Abortoutput(sockfd, "returned message too large");
+	       Die(sockfd, 504, "returned message too large");
      }
      
      readSearchResponseAPDU(&query_response, response_message + HEADER_LENGTH);
@@ -808,7 +827,7 @@ Fetchdocid(sockfd, cmd)
 
      DocIdString = strchr(inputline, ':');
      if (DocIdString == NULL) {
-	  Abortoutput(sockfd, "Malformed docid");
+	  Die(sockfd, 404, "Malformed docid");
 	  return;
      }
      else {
@@ -840,14 +859,10 @@ Fetchdocid(sockfd, cmd)
 
 
      else if ((connection=connect_to_server(server_name,atoi(service))) == NULL) 
-     {
-	  char Errorstr[256];
+	  
+	  Die(sockfd, 504, "Error openning connection to %s via service %s.", 
+	      server_name, service);
 
-	  sprintf(Errorstr,"Error openning connection to %s via service %s.", 
-		  server_name, service);
-	  Abortoutput(sockfd, Errorstr);
-	  return;
-     }
      request_message = (char*)s_malloc((size_t)MAX_MESSAGE_LEN * sizeof(char));
      response_message = (char*)s_malloc((size_t)MAX_MESSAGE_LEN * sizeof(char));
 
@@ -999,7 +1014,7 @@ SearchRemoteWAIS(sockfd, SourceName, cmd, view)
   CMDobj *cmd;
   char   *view;
 {
-     Abortoutput(sockfd, "No wais stuff in the server!!");
+     Die(sockfd, 502, "No wais stuff in the server!!");
 }
 
 
@@ -1008,7 +1023,7 @@ Fetchdocid(sockfd, cmd)
   int sockfd;
   CMDobj *cmd;
 {
-     Abortoutput(sockfd, "No wais stuff in the server!!");
+     Die(sockfd, 502, "No wais stuff in the server!!");
 }
 
 #endif
