@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.65
- * 1994/12/05 22:39:55
- * /home/mudhoney/GopherSrc/CVS/gopher+/object/GSgopherobj.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.85 $
+ * $Date: 1995/11/10 22:26:26 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/object/GSgopherobj.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,67 @@
  * Implement gopher directory functions.
  *********************************************************************
  * Revision History:
- * GSgopherobj.c,v
+ * $Log: GSgopherobj.c,v $
+ * Revision 3.85  1995/11/10  22:26:26  lindner
+ * Fix for wierd relative gopher URLs
+ *
+ * Revision 3.84  1995/11/03  18:00:45  lindner
+ * Coen: changes
+ *
+ * Revision 3.83  1995/11/03  05:37:13  lindner
+ * Convert ------ to <HR>
+ *
+ * Revision 3.82  1995/10/04  22:30:45  lindner
+ * Add <BR> to the end of info lines
+ *
+ * Revision 3.81  1995/09/28  22:59:25  lindner
+ * Move nullword
+ *
+ * Revision 3.80  1995/09/25  22:07:18  lindner
+ * Ansification
+ *
+ * Revision 3.79  1995/08/29  07:13:06  lindner
+ * Add tickets to urls
+ *
+ * Revision 3.78  1995/06/12  15:33:41  lindner
+ * Add onlyhtml support
+ *
+ * Revision 3.77  1995/05/02  04:03:51  lindner
+ * Remove unused variable
+ *
+ * Revision 3.76  1995/04/15  07:07:19  lindner
+ * Fix for fcn prototype, check for error
+ *
+ * Revision 3.75  1995/02/22  05:30:25  lindner
+ * reintroduce gsinit to gsnew
+ *
+ * Revision 3.74  1995/02/17  18:29:37  lindner
+ * Abstract display support
+ *
+ * Revision 3.73  1995/02/16  22:32:44  lindner
+ * HTML icon support
+ *
+ * Revision 3.72  1995/02/07  07:12:55  lindner
+ * oops!
+ *
+ * Revision 3.71  1995/02/07  07:06:58  lindner
+ * performance fixes
+ *
+ * Revision 3.70  1995/02/06  22:13:52  lindner
+ * Performance fixes
+ *
+ * Revision 3.69  1995/02/02  17:14:46  lindner
+ * Fix for memory leaks and accesses
+ *
+ * Revision 3.68  1995/02/01  22:08:02  lindner
+ * Put back GSaddmerge
+ *
+ * Revision 3.67  1995/02/01  21:44:41  lindner
+ * Remove GSmerge fcns, Add message/rfc822 and application/gopher to GSisText
+ *
+ * Revision 3.66  1994/12/15  17:30:49  lindner
+ * Allow multi-line abstracts in link files, Fix for ftp URL generation
+ *
  * Revision 3.65  1994/12/05  22:39:55  lindner
  * Fix Name and Path settings in GSfromURL
  *
@@ -268,6 +328,7 @@ GSnew()
      GopherObj *temp;
 
      temp = (GopherObj *) malloc(sizeof(GopherObj));
+
      temp->Selstr    = STRnew();
      temp->Title     = STRnew();
      temp->Host      = STRnew();
@@ -276,18 +337,20 @@ GSnew()
      temp->gplus     = NULL;
      temp->isask     = FALSE;
      temp->url       = NULL;
+
      GSinit(temp);
+
      return(temp);
 }
 
+
 /*
  * Initialize the gopherplus components of the object
  * (Only called for a gplus item)
  */
 
 void
-GSplusnew(gs)
-  GopherObj *gs;
+GSplusnew(GopherObj *gs)
 {
      if (gs->gplus == NULL) {
 	  gs->gplus = (GplusObj *) malloc(sizeof(GplusObj));
@@ -305,8 +368,7 @@ GSplusnew(gs)
 /*** Destroy gopher object ***/
 
 void
-GSdestroy(gs)
-  GopherObj *gs;
+GSdestroy(GopherObj *gs)
 {
      STRdestroy(gs->Selstr);
      STRdestroy(gs->Title);
@@ -326,8 +388,7 @@ GSdestroy(gs)
 /*** Destroy Gopher+ attributes ***/
 
 void
-GSplusdestroy(gs)
-  GopherObj *gs;
+GSplusdestroy(GopherObj *gs)
 {
      if (gs->gplus != NULL) {
 	  STRdestroy(gs->gplus->Admin);
@@ -348,8 +409,7 @@ GSplusdestroy(gs)
  */
 
 void
-GSinit(gs)
-  GopherObj *gs;
+GSinit(GopherObj *gs)
 {
      GSsetType(gs, '\0');
      
@@ -381,8 +441,7 @@ GSinit(gs)
  */
 
 void
-GSplusInit(gs)
-  GopherObj *gs;
+GSplusInit(GopherObj *gs)
 {
      if (gs->gplus != NULL) {
 	  STRinit(gs->gplus->Admin);
@@ -393,6 +452,7 @@ GSplusInit(gs)
      }
 }
 
+
 /* 
  * Set a URL for the gopherobject..
  */
@@ -409,8 +469,7 @@ GSsetURL(gs, url)
 }
 
 int
-GSgetNumBlocks(gs)
-  GopherObj *gs;
+GSgetNumBlocks(GopherObj *gs)
 {
      if (gs->gplus == NULL)
 	  return(-1);
@@ -419,11 +478,9 @@ GSgetNumBlocks(gs)
 }
 
 Blockobj*
-GSgetBlock(gs, bnum)
-  GopherObj *gs;
-  int bnum;
+GSgetBlock(GopherObj *gs, int bnum)
 {
-     if (gs->gplus == NULL)
+     if (gs->gplus == NULL || bnum < 0)
 	  return(NULL);
 
      return(BLAgetEntry(gs->gplus->OtherBlocks, bnum));
@@ -434,8 +491,7 @@ GSgetBlock(gs, bnum)
  */
 
 int
-GSgetNumViews(gs)
-  GopherObj *gs;
+GSgetNumViews(GopherObj *gs)
 {
      if (gs->gplus == NULL)
 	  return(0);
@@ -448,9 +504,7 @@ GSgetNumViews(gs)
  */
 
 VIewobj *
-GSgetView(gs, viewnum)
-  GopherObj *gs;
-  int        viewnum;
+GSgetView(GopherObj *gs, int viewnum)
 {
      if (gs->gplus == NULL)
 	  return(NULL);
@@ -460,22 +514,21 @@ GSgetView(gs, viewnum)
 
 
 char *
-GSgetURL(gs)
-  GopherObj *gs;
+GSgetURL(GopherObj *gs, char *ticket)
 {
      if (gs->url == NULL) {
 	  gs->url = URLnew();
-	  URLfromGS(gs->url, gs);
+	  URLfromGS(gs->url, gs, ticket);
      }
      
      return(URLget(gs->url));
 }
 
+
 char *
-GSgetURLhtml(gs)
-  GopherObj *gs;
+GSgetURLhtml(GopherObj *gs, char *ticket)
 {
-     char *cp = GSgetURL(gs);
+     char *cp = GSgetURL(gs, ticket);
      char *cp2;
      int   views;
 
@@ -491,10 +544,40 @@ GSgetURLhtml(gs)
 			 cp2 = strchr(cp+10, '/');
 			 if (cp2 != NULL) {
 			      *(++cp2) = 'h';
-			      *(++cp2) = 'h';
+			      cp2++;
+			      if (strncmp(cp2, "%2a",3)==0) {
+				   /** validated item, skip forward... **/
+				   cp2 = strstr(cp2, "%20");
+				   if (cp2 != NULL) {
+					cp2 +=3;
+					cp2 = strstr(cp2, "%20");
+					if (cp2 != NULL) {
+					     cp2+=3;
+					     *cp2 = 'h';
+					}
+				   }
+			       } else
+				    *cp2 = 'h';
      			 }
 		     }
 	       }
+	  } 
+	  if (strstr(cp, "/1validate%201") != NULL) {
+	       cp = strdup(cp);
+	       cp2 = strchr(cp+10, '/');
+	       if (cp2 != NULL) {
+		    *(++cp2) = 'h';
+		    *(++cp2) = 'h';
+		    *(cp2+11) = 'h';
+	       }
+	  }
+	  else if (strstr(cp, "/hhalidate%201") != NULL) {
+	       cp = strdup(cp);
+	       cp2 = strchr(cp+10, '/');
+	       *(cp2+13) = 'h';
+	  } else if (*(GSgetPath(gs)) == '*') {
+	       /** This is a validated item.. **/
+	       ;
 	  }
      }
 
@@ -502,10 +585,8 @@ GSgetURLhtml(gs)
 
 }
 
-char *
-GStoNetURL(gs,url)
-  GopherObj *gs;
-  char *url;
+static char *
+GStoNetURL(GopherObj *gs, char *url,  char *ticket)
 {
      char *path, *ftphost, *ftppath;
 
@@ -513,8 +594,14 @@ GStoNetURL(gs,url)
      path  = GSgetPath(gs);
 
      /** This is a server specific hack for now..  **/
+     if (path == NULL)
+	  return(NULL);
 
-     if ( (path != NULL) && (strncmp(path, "ftp:", 4) == 0) ) {
+     if ((path != NULL) && (strncmp(path, "ftp://", 6) == 0)) {
+	  strcpy(url, path);
+	  return(url);
+     }
+     else if ((path != NULL) && (strncmp(path, "ftp:", 4) == 0) ) {
 	  ftphost = path+4;
 	  ftppath = strchr(ftphost, '@');
 
@@ -532,34 +619,13 @@ GStoNetURL(gs,url)
 	       return(url);
 	  }
      }
-     strcpy(url,GSgetURLhtml(gs));
+     strcpy(url,GSgetURLhtml(gs,ticket));
      return(url);
 }
 
 
-void
-GSsetstringAsk(gs,ask)
-  GopherObj *gs;
-  char *ask;
-{
-     GSsetBlock(gs,"ASK",ask,TRUE);
-     GSsetAsk(gs,TRUE);
-}
-
-
-void
-GSsetAbstract(gs, abstract)
-  GopherObj *gs;
-  char *abstract;
-{
-     GSsetBlock(gs,"ABSTRACT",abstract,TRUE);
-}
-
-
 Blockobj*
-GSfindBlock(gs, blockname)
-  GopherObj *gs;
-  char *blockname;
+GSfindBlock(GopherObj *gs, char *blockname)
 {
      int x;
      
@@ -573,11 +639,7 @@ GSfindBlock(gs, blockname)
 }
 
 void
-GSsetBlock(gs, blockname, text, appendIfPoss)
-  GopherObj *gs;
-  char *blockname;
-  char *text;
-  boolean appendIfPoss;
+GSsetBlock(GopherObj *gs, char *blockname, char *text, boolean appendIfPoss)
 {
      Blockobj *bl = NULL;
      boolean newbl = FALSE;
@@ -598,7 +660,7 @@ GSsetBlock(gs, blockname, text, appendIfPoss)
      
 #ifdef DEBUGGING
      if (DEBUG)
-	  BLtoNet(bl, fileno(stderr));
+	  BLtoNet(bl, fileno(stderr), TRUE);
 #endif
      if (gs->gplus == NULL)
 	  GSplusnew(gs);
@@ -615,9 +677,7 @@ GSsetBlock(gs, blockname, text, appendIfPoss)
  */
 
 char **
-GSsetAskdata(gs, askdata)
-  GopherObj *gs;
-  char **askdata;
+GSsetAskdata(GopherObj *gs, char **askdata)
 {
      if (!GSgplusInited(gs))
 	  return(NULL);
@@ -648,8 +708,7 @@ GSsetAskdata(gs, askdata)
 
 /* Converts the gopher+ moddate from <YYYYMMDDHHMMSS> to a struct tm ptr */
 struct tm *
-GSgetModDateTM(gs)
-     GopherObj *gs ;
+GSgetModDateTM(GopherObj *gs)
 {
      static struct tm time;
      char *cp;
@@ -694,8 +753,7 @@ GSgetModDateTM(gs)
  *  (The i18n definition's are turned on or off in Locale.h)
  */
 char *
-GSgetModDate(gs)
-  GopherObj *gs;
+GSgetModDate(GopherObj *gs)
 {
      char	    dateBuf[256];
      char	    *datetime;
@@ -723,9 +781,7 @@ GSgetModDate(gs)
 
 
 void
-GSsetAdmin(gs, admin)
-  GopherObj *gs;
-  char *admin;
+GSsetAdmin(GopherObj *gs, char *admin)
 {
      if (gs->gplus == NULL)
 	  GSplusnew(gs);
@@ -736,9 +792,7 @@ GSsetAdmin(gs, admin)
 
 
 void
-GSsetModDate(gs, str)
-  GopherObj *gs;
-  char *str;
+GSsetModDate(GopherObj *gs, char *str)
 {
      if (gs->gplus == NULL) GSplusnew(gs);
      STRset(gs->gplus->ModDate, str);
@@ -749,11 +803,7 @@ GSsetModDate(gs, str)
 /** Add a view for a specific gopherobj **/
 
 void
-GSaddView(gs, view, language, estsize)
-  GopherObj *gs;
-  char *view;
-  char *language;
-  int estsize;
+GSaddView(GopherObj *gs, char *view, char *language, int estsize)
 {
      char tmpstr[64];
      VIewobj *temp;
@@ -773,12 +823,15 @@ GSaddView(gs, view, language, estsize)
      VIdestroy(temp);
 }
 
+void
+GSdelTopView(GopherObj *gs)
+{
+#define VIApop(a) (DApop((DynArray*)(a)))
+     VIApop(gs->gplus->Views);
+}
 
 void
-GSaddBlock(gs, Blockname, file)
-  GopherObj *gs;
-  char *Blockname;
-  char *file;
+GSaddBlock(GopherObj *gs, char *Blockname, char *file)
 {
      Blockobj *bl;
 
@@ -790,6 +843,8 @@ GSaddBlock(gs, Blockname, file)
 	  GSplusnew(gs);
 
      BLApush(gs->gplus->OtherBlocks, bl);
+
+     BLdestroy(bl);
 }
 
 
@@ -798,14 +853,14 @@ GSaddBlock(gs, Blockname, file)
  * Send a gopher reference into an fd
  */
 
+static char *nullword = "(NULL)";
+
 void
-GStoNet(gs, sockfd, fmt, ticket)
-  GopherObj *gs;
-  int       sockfd;
-  GSformat  fmt;
-  char      *ticket;
+GStoNet(GopherObj *gs, int sockfd, GSformat fmt, char *ticket)
 {
      char buf[1024];
+     int i;
+     Blockobj *bl;
 
      if (ticket == NULL)
 	  ticket = "";
@@ -816,11 +871,16 @@ GStoNet(gs, sockfd, fmt, ticket)
 	buf[0] = '3'; 
 
      if (fmt == GSFORM_G0) {
+	  for (i=0; i< GSgetNumBlocks(gs); i++) {
+	       bl = GSgetBlock(gs, i);
+	       if (strcasecmp(BLgetName(bl), "ONLYHTML")==0)
+		    return;
+	  }
 	  sprintf(buf + 1, "%s\t%s%s\t%s\t%d",
-		  GSgetTitle(gs) ? GSgetTitle(gs) : "(NULL)",
-		  ticket         ? ticket : "(NULL)",
-		  GSgetPath(gs)  ? GSgetPath(gs) : "(NULL)",
-		  GSgetHost(gs)  ? GSgetHost(gs) : "(NULL)",
+		  GSgetTitle(gs) ? GSgetTitle(gs) : nullword,
+		  ticket         ? ticket : nullword,
+		  GSgetPath(gs)  ? GSgetPath(gs) : nullword,
+		  GSgetHost(gs)  ? GSgetHost(gs) : nullword,
 		  GSgetPort(gs));
 	  
 	  if (GSisAsk(gs))
@@ -838,38 +898,41 @@ GStoNet(gs, sockfd, fmt, ticket)
      }
      else if (fmt == GSFORM_HTML) {
 	  char url[256];
+	  for (i=0; i < GSgetNumBlocks(gs); i++) {
+	       bl = GSgetBlock(gs, i);
+	       if (strcmp(BLgetName(bl), "NOHTML") == 0)
+		    return;
+	  }
 
 	  if (GSgetType(gs) == A_INFO) {
 	       writestring(sockfd, "<DT>");
-	       writestring(sockfd, GSgetTitle(gs));
+	       if (strncmp(GSgetTitle(gs), "----------------",16)==0) {
+		    writestring(sockfd, "<HR>");
+	       } else
+		    writestring(sockfd, GSgetTitle(gs));
+	       writestring(sockfd, "<BR>");
 	       return;
 	  }
-	  GStoNetURL(gs, url);
+	  GStoNetURL(gs, url, ticket);
 	  if (url[0] == '\0')
 	       return;
-	  sprintf(buf, "<DT><A HREF=\"%s\">%s</A>\r\n", url, GSgetTitle(gs));
+	  sprintf(buf, "<A HREF=\"%s\">%s</A>\r\n", url, GSgetTitle(gs));
 	  writestring(sockfd, buf);
 
 	  if (GSgplusInited(gs)) {
-	       int  i, j;
 	       Blockobj *bl;
 
-	       for (i=0; i<GSgetNumBlocks(gs); i++) {
-		    if (strcmp(BLgetName(GSgetBlock(gs, i)),"ABSTRACT")==0){
-			 bl = GSgetBlock(gs, i);
-			 writestring(sockfd, "<DD>");
-			 for (j=0; j < BLgetNumLines(bl); j++) {
-			      sprintf(buf, "%s\r\n", BLgetLine(bl, j));
-			      writestring(sockfd, buf);
-			 }
-			 writestring(sockfd, "\r\n");
-		    }
+	       bl = GSfindBlock(gs, "ABSTRACT");
+	       if (bl != NULL) {
+		    writestring(sockfd, "<DD>");
+		    BLtoNet(bl, sockfd, FALSE);
+		    writestring(sockfd, "\r\n");
 	       }
 	  }
 
 	  if (GSgetWeight(gs) != 0) {
 	       sprintf(buf, "<DD>Score: %d\r\n", GSgetWeight(gs));
-	       writestring(buf);
+	       writestring(sockfd, buf);
 	  }
      }
 	  
@@ -883,15 +946,16 @@ GStoNet(gs, sockfd, fmt, ticket)
  */
 
 void
-GSplustoNet(gs, sockfd, filter, ticket)
-  GopherObj *gs;
-  int sockfd;
-  char **filter;
-  char *ticket;
+GSplustoNet(GopherObj *gs, int sockfd, char **filter, char *ticket)
 {
-     int     i;
-     char    tmpstr[256];
-     boolean sendviews, sendadmin, sendothers;
+     int      i;
+     char     tmpstr[256];
+     boolean  sendviews, sendadmin, sendothers;
+
+     for (i=0; i < GSgetNumBlocks(gs); i++) {
+	  if (strcmp(BLgetName(GSgetBlock(gs, i)), "ONLYHTML") == 0)
+	       return;
+     }
 
      if (filter == NULL)
 	  sendviews = sendadmin = sendothers = TRUE;
@@ -915,10 +979,10 @@ GSplustoNet(gs, sockfd, filter, ticket)
 
      /** Only write out "interesting" URLs **/
 
-     if (GSgetURL(gs) != NULL) {
-	  if (strncmp(GSgetURL(gs), "gopher:", 7) != 0) {
+     if (GSgetURL(gs,ticket) != NULL) {
+	  if (strncmp(GSgetURL(gs,ticket), "gopher:", 7) != 0) {
 	       writestring(sockfd, "+URL:\r\n ");
-	       writestring(sockfd, GSgetURL(gs));
+	       writestring(sockfd, GSgetURL(gs,ticket));
 	       writestring(sockfd, "\r\n");
 	  }
      }
@@ -951,7 +1015,7 @@ GSplustoNet(gs, sockfd, filter, ticket)
 	  if (sendothers) {
 
 	       for (i=0; i< GSgetNumBlocks(gs); i++)
-		    BLtoNet(GSgetBlock(gs, i),sockfd);
+		    BLtoNet(GSgetBlock(gs, i),sockfd, TRUE);
 	  }
      }
 }
@@ -967,9 +1031,7 @@ GSplustoNet(gs, sockfd, filter, ticket)
  ***/
 
 int
-GSplusfromNet(gs, fd)
-  GopherObj *gs;
-  int fd;
+GSplusfromNet(GopherObj *gs, int fd)
 {
      int result,readok,i;
      boolean nextinfo = FALSE;
@@ -1021,7 +1083,8 @@ GSplusfromNet(gs, fd)
 
 	       if (strcasecmp(inputline, "INFO")==0) {
 		    /** Get rid of the space. **/
-		    readrecvbuf(fd, &plusfield,1);
+		    if (readrecvbuf(fd, &plusfield,1) <= 0)
+			 return(HARDERROR);
 		    BLdestroy(bl);
 		    if (readok == SOFTERROR)
 			 return(SOFTERROR);
@@ -1111,10 +1174,24 @@ GSplusfromNet(gs, fd)
 extern int readfield();
 extern int readline();
 
+static void
+GSprocessObject(GopherObj *gs)
+{
+ 	if (DEBUG)
+		return;
+#ifdef GOPHERTOFTP_HOST
+    if (strncmp(GSgetPath(gs),"ftp:",4) == 0) {
+	GSsetPort(gs,GOPHERTOFTP_PORT);
+ 	GSsetHost(gs,GOPHERTOFTP_HOST);
+     }
+#endif
+}
+
+
+
+
 int
-GSfromNet(gs, sockfd)
-  GopherObj *gs;
-  int sockfd;
+GSfromNet(GopherObj *gs, int sockfd)
 {
      char foo[1024], *cp;
      
@@ -1148,18 +1225,18 @@ GSfromNet(gs, sockfd)
        case A_MOVIE:
        case A_APP:
 	  break;
-	   case A_ERROR:
-		GSsetPath(gs, "");
-		GSsetHost(gs, "");
-	  	GSsetGplus(gs, FALSE);
-	    ZapCRLF(foo+1);
-	    cp = foo + strlen(foo+1);
-	    while (*cp == '.' && (*(cp-1) == '\n' || *(cp-1) == '\r')) {
-			*cp = '\0';
-			ZapCRLF(foo+1);
-		}
-		break;
-       case A_EOI:
+     case A_ERROR:
+	  GSsetPath(gs, "");
+	  GSsetHost(gs, "");
+	  GSsetGplus(gs, FALSE);
+	  ZapCRLF(foo+1);
+	  cp = foo + strlen(foo+1);
+	  while (*cp == '.' && (*(cp-1) == '\n' || *(cp-1) == '\r')) {
+	       *cp = '\0';
+	       ZapCRLF(foo+1);
+	  }
+	  break;
+     case A_EOI:
 	  if (foo[1] == '\r' && foo[2] == '\n')
 	       return(1);
      default:
@@ -1212,8 +1289,7 @@ GSfromNet(gs, sockfd)
 /** Copy a GopherObj ***/
 
 void
-GScpy(dest, orig)
-  GopherObj *dest, *orig;
+GScpy(GopherObj *dest, GopherObj *orig)
 {
      dest->sFileType = orig->sFileType;
      dest->iPort     = orig->iPort;
@@ -1225,7 +1301,8 @@ GScpy(dest, orig)
      
      GSsetGplus(dest, GSisGplus(orig));
      GSsetAsk(dest, GSisAsk(orig));
-     GSsetURL(dest, GSgetURL(orig));
+     if (orig->url != NULL)
+	  GSsetURL(dest, GSgetURL(orig, ""));
      GSsetTTL(dest, GSgetTTL(orig));
 
      STRinit(dest->Localfile);
@@ -1235,8 +1312,7 @@ GScpy(dest, orig)
 }
 
 void
-GSpluscpy(dest, orig)
-  GopherObj *dest, *orig;
+GSpluscpy(GopherObj *dest, GopherObj *orig)
 {
      if (GSgplusInited(orig)) {
 	  if (!GSgplusInited(dest))
@@ -1253,16 +1329,17 @@ GSpluscpy(dest, orig)
 /* Called by GDaddGSmerge to merge gs into directory, that is only called 
    by function to do this from a link */
 void
-GSmerge(gs, overlay)
-  GopherObj *gs, *overlay;
+GSmerge(GopherObj *gs, GopherObj *overlay)
 {
      char *tempstr;
+     char oldFileType;
      
      DebugGSplusPrint(gs,"GSmerge: Original");
      DebugGSplusPrint(overlay,"GSmerge: Overlay");
      
      if (GSgetHost(overlay) != NULL) {
-	  
+	  oldFileType = GSgetType(gs);
+
 	  if (GSgetType(overlay) != '\0') {
 	       /* Just setting Type wont work, since they interelated with Path
 	        * so set first char of path as well */
@@ -1292,25 +1369,50 @@ GSmerge(gs, overlay)
 	       GSsetNum(gs, GSgetNum(overlay));
 	  if (GSgetWeight(overlay) != 0)
 	       GSsetWeight(gs, GSgetWeight(overlay));
-	  if (GSgetAdmin(overlay) != NULL)
-               GSsetAdmin(gs, GSgetAdmin(overlay));
-          if (overlay->gplus != NULL)
-          {
+
+	  if (GSgplusInited(overlay)) {
 	       if (!GSgplusInited(gs))
-                    GSplusnew(gs);
-               BLAcpy(gs->gplus->OtherBlocks,
+		    GSplusnew(gs);
+	       BLAcpy(gs->gplus->OtherBlocks,
 		      overlay->gplus->OtherBlocks);
 	  }
+
+	  if (GSgetAdmin(overlay) != NULL)
+               GSsetAdmin(gs, GSgetAdmin(overlay));
+
+
+ 	  if (GSgetModDate(overlay) != NULL)
+	       GSsetModDate(gs, GSgetModDate(overlay));
+ 	  if (GSgetNumViews(gs) && oldFileType == A_FILE
+	      && GSgetType(gs) != A_FILE) {
+	       char *vs, vsize[64], *vc, vcomments[256];
+	       VIewobj *temp;
+	       if (vs = VIgetSize(GSgetView(gs, 0)))
+		    strcpy(vsize, vs);
+	       if (vc = VIgetComments(GSgetView(gs, 0)))
+		    strcpy(vcomments, vc);
+	       VIAdestroy(gs->gplus->Views);
+	       gs->gplus->Views    = VIAnew(10);
+	       temp = VInew();
+	       VIsetType(temp, "");
+	       VIsetLang(temp, "");
+	       VIsetSize(temp, vsize);
+	       VIsetComments(temp, vcomments);
+	       VIApush(gs->gplus->Views, temp);
+	       VIdestroy(temp);
+ 	  }
+ 	  if (GSgetNumViews(overlay))
+	       BLAcpy(gs->gplus->Views, overlay->gplus->Views);
      }
+     
      DebugGSplusPrint(gs,"GSmerge: Result");
 }
 
 
 /** Compare two GopherObjs ***/
 
-int
-GScmp(gs1, gs2)
-  GopherObj *gs1, *gs2;
+static int
+GScmp(GopherObj *gs1, GopherObj *gs2)
 {
      if (GSgetTitle(gs1) == NULL)
 	  return(1);
@@ -1339,8 +1441,7 @@ GScmp(gs1, gs2)
  */
 
 int
-GSconnect(gs)
-  GopherObj *gs;
+GSconnect(GopherObj *gs)
 {
      int sockfd; 
 
@@ -1362,12 +1463,7 @@ GSconnect(gs)
  */
 
 void 
-GStransmit(gs, sockfd, search, command, view)
-  GopherObj *gs;
-  int sockfd;
-  char *search;
-  char *command;
-  char *view;
+GStransmit(GopherObj *gs, int sockfd, char *search, char *command, char *view)
 {
      char *cp;
      char **ask = GSgetAskdata(gs);
@@ -1420,9 +1516,7 @@ GStransmit(gs, sockfd, search, command, view)
  *
  */
 void
-GSsendHeader(sockfd, size)
-  int sockfd;
-  int size;
+GSsendHeader(int sockfd, int size)
 {
      char sizestr[64];
 
@@ -1433,11 +1527,7 @@ GSsendHeader(sockfd, size)
 
 /** GSsendErrorHeader sends an error message header/message to the client **/
 void
-GSsendErrorHeader(gs,sockfd,errortype,errormsg)
-  GopherObj *gs;
-  int       sockfd;
-  int       errortype;
-  char      *errormsg;
+GSsendErrorHeader(GopherObj *gs, int sockfd, int errortype, char *errormsg)
 {
      char tmpstr[512];
 
@@ -1461,9 +1551,7 @@ GSsendErrorHeader(gs,sockfd,errortype,errormsg)
  */
 
 int
-GSrecvHeader(gs, sockfd)
-  GopherObj *gs;
-  int sockfd;
+GSrecvHeader(GopherObj *gs, int sockfd)
 {
      char headerline[256];
 
@@ -1510,9 +1598,7 @@ GSrecvHeader(gs, sockfd)
  */
 
 void
-GSgetginfo(gs, savename)
-  GopherObj *gs;
-  boolean   savename;
+GSgetginfo(GopherObj *gs, boolean   savename)
 {
      int    sockfd, bytes;
      char   inputline[256];
@@ -1545,7 +1631,7 @@ GSgetginfo(gs, savename)
 	  return;
      }
      
-     GStransmit(gs, sockfd, NULL, "!", NULL, NULL);
+     GStransmit(gs, sockfd, NULL, "!", NULL);
      bytes = GSrecvHeader(gs,sockfd);
 
      if (bytes == 0)
@@ -1577,13 +1663,13 @@ GSgetginfo(gs, savename)
  */
 
 int
-GSfromLink(gs, fio, host, port, directory, peer)
-  GopherObj *gs;
-  FileIO    *fio;
-  char      *host;
-  int       port;
-  char      *directory;
-  char      *peer;
+GSfromLink(
+  GopherObj *gs,
+  FileIO    *fio,
+  char      *host,
+  int       port,
+  char      *directory,
+  char      *peer)
 {
      int doneflags = 0;
      char buf[1024];
@@ -1607,7 +1693,6 @@ GSfromLink(gs, fio, host, port, directory, peer)
 
 	  ZapCRLF(buf);
 
-
 	  if (strncmp(buf, "Type=", 5)==0) {
 	       GSsetType(gs, buf[5]);
 
@@ -1662,8 +1747,20 @@ GSfromLink(gs, fio, host, port, directory, peer)
 	  else if (strncmp(buf, "Numb=", 5)==0)
 	       GSsetNum(gs, atoi(buf+5));
 
-	  else if (strncmp(buf, "Abstract=", 9)==0)
-	       GSsetAbstract(gs, buf+9);
+	  else if (strncmp(buf, "Abstract=", 9)==0) {
+	       char *acp;
+
+	       acp = buf+9;
+
+	       while (*(buf + strlen(buf)-1) == '\\' && bytesread >0) {
+		    /* A continuation line */
+		    *(buf + strlen(buf)-1) = '\0';
+		    GSsetAbstract(gs, acp);
+		    bytesread = FIOreadlinezap(fio, buf, sizeof(buf));
+		    acp = buf;
+	       }
+	       GSsetAbstract(gs, acp);
+	  }
 	  else if (strncmp(buf, "Admin=", 6) == 0)
 	       GSsetAdmin(gs, buf +6);
 	  else if (strncmp(buf, "URL=", 4) == 0)
@@ -1756,12 +1853,7 @@ GSfromLink(gs, fio, host, port, directory, peer)
 
  
 int
-GSfromURL(gs, urltxt, host, port, doneflags)
-  GopherObj *gs;
-  char	    *urltxt;
-  char      *host;
-  int        port;
-  int        doneflags;
+GSfromURL(GopherObj *gs, char *urltxt, char *host, int port, int doneflags)
 {
      char tempbuf[256];
      Url  *url;
@@ -1849,7 +1941,7 @@ GSfromURL(gs, urltxt, host, port, doneflags)
      if (serviceType == ftp) {
 	  if (!(doneflags & G_PATH)) {
 	       if (URLgetPath(url) != NULL && *URLgetPath(url) != '\0')
-		    sprintf(tempbuf, "ftp:%s@%s", URLgetHost(url),
+		    sprintf(tempbuf, "ftp:%s@/%s", URLgetHost(url),
 			    URLgetPath(url));
 	       else
 		    sprintf(tempbuf, "ftp:%s@/", URLgetHost(url));
@@ -1891,10 +1983,7 @@ GSfromURL(gs, urltxt, host, port, doneflags)
 
 
 void
-GStoLink(gs, fd, AddInfo)
-  GopherObj *gs;
-  int fd;
-  BOOLEAN AddInfo;
+GStoLink(GopherObj *gs, int fd, BOOLEAN AddInfo)
 {
      char gtype[2];
      char portnum[16];
@@ -1928,18 +2017,18 @@ GStoLink(gs, fd, AddInfo)
 
 
 boolean
-GSisText(gs, view)
-  GopherObj *gs;
-  char *view;
+GSisText(GopherObj *gs, char *view)
 {
-
      if (view == NULL) {
 	  switch (GSgetType(gs)) {
+	  case A_DIRECTORY:
 	  case A_FILE:
+	  case A_MIME:
+	  case A_CSO:
 	  case A_MACHEX:
-	  case A_HTML:
 	       return(TRUE);
 
+	  case A_HTML:		/* For goofy relative urls, ugh! */
 	  default:
 	       return(FALSE);
 	  }
@@ -1954,9 +2043,12 @@ GSisText(gs, view)
 	  }
 
 	  if (strncasecmp(view, "Text",4) == 0 ||
+	      strncasecmp(view, "message/rfc822", 14)==0 ||
 	      strncasecmp(view, "application/postscript", 21)==0 ||
 	      strncasecmp(view, "application/mac-binhex40", 24)==0 ||
-	      strncasecmp(view, "application/rtf", 15) == 0)
+	      strncasecmp(view, "application/rtf", 15) == 0 ||
+	      strncasecmp(view, "application/gopher", 18) == 0)
+
 
 
 	       return(TRUE);
@@ -1967,9 +2059,7 @@ GSisText(gs, view)
 
 #ifdef DEBUGGING
 void
-GSplusPrint(gs,head)
-  GopherObj *gs;
-  char *head;
+GSplusPrint(GopherObj *gs, char *head)
 {
      int i;
      int oldDebug = DEBUG;
@@ -1989,7 +2079,7 @@ GSplusPrint(gs,head)
      
      if (GSgplusInited(gs))
 	  for (i=0; i< GSgetNumBlocks(gs); i++) {
-	       BLtoNet(GSgetBlock(gs, i), fileno(stderr));
+	       BLtoNet(GSgetBlock(gs, i), fileno(stderr), TRUE);
 	  }
      fprintf(stderr,"===============\r\n");
      DEBUG = oldDebug;
