@@ -1,9 +1,9 @@
 /********************************************************************
- * lindner
- * 3.31
- * 1994/12/03 02:14:13
- * /home/mudhoney/GopherSrc/CVS/gopher+/object/fileio.c,v
- * Exp
+ * $Author: lindner $
+ * $Revision: 3.41 $
+ * $Date: 1996/01/04 18:25:12 $
+ * $Source: /home/arcwelder/GopherSrc/CVS/gopher+/object/fileio.c,v $
+ * $State: Exp $
  *
  * Paul Lindner, University of Minnesota CIS.
  *
@@ -14,7 +14,40 @@
  * Socket/file input output routines.
  *********************************************************************
  * Revision History:
- * fileio.c,v
+ * $Log: fileio.c,v $
+ * Revision 3.41  1996/01/04  18:25:12  lindner
+ * Updates for autoconf
+ *
+ * Revision 3.40  1995/11/03  18:01:28  lindner
+ * Fix for waitpid stuff
+ *
+ * Revision 3.39  1995/09/25  22:07:23  lindner
+ * Ansification
+ *
+ * Revision 3.38  1995/06/12  15:32:41  lindner
+ * Close correct file descriptor
+ *
+ * Revision 3.37  1995/06/08  05:17:54  lindner
+ * From Fote: Deal with UCX's funny EPIPE return value on EOF
+ * immediately after each socket_read().
+ *
+ * Ensure that file is closed on fstat() error in FIOopenUFS()
+ *
+ * Revision 3.36  1995/04/15  06:53:39  lindner
+ * Different way of returning errors
+ *
+ * Revision 3.35  1995/03/09  05:28:19  lindner
+ * Insure files get closed in FIO routines
+ *
+ * Revision 3.34  1995/02/27  17:45:52  lindner
+ * Fix for sco
+ *
+ * Revision 3.33  1995/01/02  21:09:46  lindner
+ * Fix for UCX menu ordering
+ *
+ * Revision 3.32  1994/12/20  17:09:46  lindner
+ * Fix for binary file retrievals
+ *
  * Revision 3.31  1994/12/03  02:14:13  lindner
  * Correct fix for waitpid() code
  *
@@ -126,6 +159,7 @@
 #include "Wait.h"
 #include "String.h"
 #include "Stdlib.h"
+#include "compatible.h"
 
 #ifdef VMS
 #  include <stat.h>
@@ -140,6 +174,9 @@
 #  endif
 #endif
 
+static char **  FIOgetargv(char *cmd);
+
+
 /*
  * Cache of used fios 
  */
@@ -163,8 +200,7 @@ static FileIO *FIOpopUsedfio()
  * Push an item on our recently used stack.
  */
  
-static boolean FIOpushUsedfio(oldfio)
-  FileIO *oldfio;
+static boolean FIOpushUsedfio(FileIO *oldfio)
 {
      if (FIOnumusedfios < FIOMAXOFILES) {
 	  FIOusedfios[FIOnumusedfios++] = oldfio;
@@ -215,8 +251,7 @@ FIOnew()
  */
 
 void
-FIOdestroy(fio)
-  FileIO *fio;
+FIOdestroy(FileIO *fio)
 {
      if (FIOpushUsedfio(fio)) {
 	  /** No space in cache. **/
@@ -229,10 +264,7 @@ FIOdestroy(fio)
  */
 
 FileIO *
-FIOopenUFS(fname, flags, mode)
-  char   *fname;
-  int    flags;
-  int    mode;
+FIOopenUFS(char *fname, int flags, int mode)
 {
      int    fd;
      FileIO *fio;
@@ -247,8 +279,10 @@ FIOopenUFS(fname, flags, mode)
      if (fd < 0)
 	  return(NULL); /* Couldn't open file */
 
-     if (fstat(fd, &buf)) 
+     if (fstat(fd, &buf)) { 
+          close(fd);
 	  return(NULL); /* Couldn't open anything */
+     }
 
      fio = FIOnew();
      
@@ -265,9 +299,7 @@ FIOopenUFS(fname, flags, mode)
  */
 
 FileIO*
-FIOopenfd(fd, issocket)
-  int     fd;
-  boolean issocket;
+FIOopenfd(int fd, boolean issocket)
 {
      FileIO *fio;
 
@@ -279,10 +311,7 @@ FIOopenfd(fd, issocket)
 }
 
 FileIO*
-FIOopenProcess(prog, args, rw)
-  char   *prog;
-  char   **args;
-  char   *rw;
+FIOopenProcess(char *prog, char **args, char   *rw)
 {
      int    pfd[2];
      int    pid;
@@ -362,8 +391,7 @@ FIOopenProcess(prog, args, rw)
  */
 
 int
-FIOclose(fio)
-  FileIO *fio;
+FIOclose(FileIO *fio)
 {
      int result;
 
@@ -402,16 +430,16 @@ FIOclose(fio)
  */
 
 int
-FIOwaitpid(fio)
-  FileIO *fio;
+FIOwaitpid(FileIO *fio)
 {
      Portawait   status;
      pid_t       result;
 
 
      do {
+	  errno = 0;
 	  result = waitpid(FIOgetPid(fio), &status, 0);
-     } while (result != FIOgetPid(fio) && (errno == EINTR));
+     } while (result != FIOgetPid(fio) && (errno == ECHILD));
 
      return(Gwaitstatus(status) & 0xf);
 }
@@ -423,10 +451,7 @@ FIOwaitpid(fio)
  */
 
 int
-FIOwriten(fio, ptr, nbytes)
-  FileIO *fio;
-  char   *ptr;
-  int    nbytes;
+FIOwriten(FileIO *fio, char *ptr, int nbytes)
 {
      int nleft, nwritten;
      int fd = FIOgetfd(fio);
@@ -452,9 +477,7 @@ FIOwriten(fio, ptr, nbytes)
  */
    
 int
-FIOwritestring(fio, str)
-  FileIO *fio;
-  char   *str;
+FIOwritestring(FileIO *fio, char *str)
 {
      int length;
 
@@ -473,11 +496,8 @@ FIOwritestring(fio, str)
 }
 
 #ifdef MMAP_IO
-int
-FIOreadbuf_mmap(fio, newbuf, newbuflen)
-  FileIO *fio;
-  char   *newbuf;
-  int     newbuflen;
+static int
+FIOreadbuf_mmap(FileIO *fio, char *newbuf, int newbuflen)
 {
      struct stat buf;
      char        *bytes;
@@ -486,8 +506,10 @@ FIOreadbuf_mmap(fio, newbuf, newbuflen)
 	  /** Do MMAP IO if we can... **/
 
 	  if (fio->buf == NULL) {
-	       if (fstat(FIOgetfd(fio), &buf))
+	       if (fstat(FIOgetfd(fio), &buf)) {
+		    close(FIOgetfd(fio));
 		    return(-1);
+	       }
 	  
 	       if (buf.st_size == 0)
 		    return(0);
@@ -498,7 +520,9 @@ FIOreadbuf_mmap(fio, newbuf, newbuflen)
 	       bytes = mmap(0, buf.st_size, PROT_READ,
 			    MAP_SHARED | MAP_FILE,
 			    FIOgetfd(fio), 0);
+#ifdef HAVE_MADVISE
 	       madvise(bytes, buf.st_size, MADV_SEQUENTIAL);
+#endif
 
 	       if (bytes == (caddr_t) -1)
 		    return(-1);
@@ -532,10 +556,7 @@ FIOreadbuf_mmap(fio, newbuf, newbuflen)
  */
 
 int
-FIOreadbuf(fio, newbuf, newbuflen)
-  FileIO *fio;
-  char   *newbuf;
-  int     newbuflen;
+FIOreadbuf(FileIO *fio, char *newbuf, int newbuflen)
 {
      int len;
      int fd = FIOgetfd(fio);
@@ -555,13 +576,17 @@ FIOreadbuf(fio, newbuf, newbuflen)
 	  
 	  len = FIOisSocket(fio) ? socket_read(fd, fio->buf, FIOBUFSIZE) :
 	       read(fd, fio->buf, FIOBUFSIZE);
-	  FIOsetBufDsize(fio, len);
-	  FIOsetBufIndex(fio, 0);
 	  
 #if defined(FIO_NOMULTIEOF)
-	  if (len < 0 && errno == EPIPE)
+	  if (len < 0 && errno == EPIPE) { /** EOF **/
+	       FIOsetBufIndex(fio,-1);
 	       return(0);
+	  }
 #endif
+
+	  FIOsetBufDsize(fio, len);
+	  FIOsetBufIndex(fio, 0);
+
 	  if (len == 0) {
 	       FIOsetBufIndex(fio,-1);
 	       return(bytesread); /** EOF **/
@@ -578,6 +603,12 @@ FIOreadbuf(fio, newbuf, newbuflen)
 	       /** Read another buffer **/
 	       len = FIOisSocket(fio) ? socket_read(fd, fio->buf, FIOBUFSIZE) :
 		    read(fd, fio->buf, FIOBUFSIZE);
+#if defined(FIO_NOMULTIEOF)
+	       if (len < 0 && errno == EPIPE) { /** EOF **/
+		    len = 0;
+		    errno = 0;
+	       }
+#endif
 
 	       if (len == 0) {
 		    FIOsetBufIndex(fio,-1);
@@ -587,7 +618,7 @@ FIOreadbuf(fio, newbuf, newbuflen)
 		    return(len);       /** Error **/
 
 	       FIOsetBufDsize(fio, len);
-	       FIOsetBufIndex(fio, -1);
+	       FIOsetBufIndex(fio, 0);
 	  } else if (FIOgetBufIndex(fio) >= FIOgetBufDsize(fio))
 	       /* Read a new buffer next time through */
 	       FIOsetBufIndex(fio, -1);
@@ -599,10 +630,7 @@ FIOreadbuf(fio, newbuf, newbuflen)
 
 /* Read 'n' bytes from a descriptor, non buffered direct into the storage. */
 int
-FIOreadn(fio, ptr, nbytes)
-  FileIO *fio;
-  char   *ptr;
-  int    nbytes;
+FIOreadn(FileIO *fio, char   *ptr, int    nbytes)
 {
      int nleft, nread;
      int fd = FIOgetfd(fio);
@@ -612,8 +640,10 @@ FIOreadn(fio, ptr, nbytes)
 	  nread = FIOisSocket(fio) ? socket_read(fd, ptr, nleft) :
 	       read(fd, ptr, nleft);
 #if defined(FIO_NOMULTIEOF)
-          if (nread < 0 && errno == EPIPE)
-	       break;
+          if (nread < 0 && errno == EPIPE) { /** EOF **/
+	       nread = 0;
+	       errno = 0;
+	  }
 #endif
 
 	  if (nread < 0)
@@ -638,10 +668,7 @@ FIOreadn(fio, ptr, nbytes)
  */
 
 int
-FIOreadline(fio, ptr, maxlen)
-  FileIO *fio;
-  char   *ptr;
-  int    maxlen;
+FIOreadline(FileIO *fio, char *ptr, int maxlen)
 {
      int bytesread;
      int rc;
@@ -660,7 +687,7 @@ FIOreadline(fio, ptr, maxlen)
 		    break;		/* EOF, some data was read */
 	  }
 	  else
-	       return(-1);		/* error */
+	       return(rc);		/* error */
      }
      
      *ptr = 0; 				/* Tack a NULL on the end */
@@ -678,10 +705,7 @@ FIOreadline(fio, ptr, maxlen)
  */
 
 int 
-FIOreadlinezap(fio, ptr, maxlen)
-  FileIO *fio;
-  char   *ptr;
-  int    maxlen;
+FIOreadlinezap(FileIO *fio, char *ptr, int maxlen)
 {
      int len;
 
@@ -689,7 +713,7 @@ FIOreadlinezap(fio, ptr, maxlen)
      ptr += len;
      ptr --;
      
-     if (*ptr == '\r') {
+     if (len && *ptr == '\r') {
 	  ptr[len] = '\0';
 	  len--;
      }
@@ -705,11 +729,7 @@ FIOreadlinezap(fio, ptr, maxlen)
  */
 
 int 
-FIOreadtoken(fio, ptr, maxlen, zechar)
-  FileIO *fio;
-  char	 *ptr;
-  int 	 maxlen;
-  char   zechar;
+FIOreadtoken(FileIO *fio, char *ptr, int maxlen, char zechar)
 {
      int bytesread;
      int rc;
@@ -731,7 +751,7 @@ FIOreadtoken(fio, ptr, maxlen, zechar)
 		    break;		/* EOF, some data was read */
 	  }
 	  else
-	       return(-1);		/* error */
+	       return(rc);		/* error */
      }
      
      *ptr = 0; 				/* Tack a NULL on the end */
@@ -739,10 +759,8 @@ FIOreadtoken(fio, ptr, maxlen, zechar)
      return(bytesread);
 }
 
-int
-FIOexecv(prog, args)
-  char *prog;
-  char **args;
+static int
+FIOexecv(char *prog, char **args)
 {
      FileIO *fio;
      int    result;
@@ -777,9 +795,8 @@ FIOexecv(prog, args)
  * Do the minimal shell/dcl processing
  */
 
-char **
-FIOgetargv(cmd)
-  char *cmd;
+static char **
+FIOgetargv(char *cmd)
 {
      int           inquote = 0;
      int           insquote = 0;
@@ -853,8 +870,7 @@ FIOgetargv(cmd)
  */
 
 int
-FIOsystem(cmd)
-  char *cmd;
+FIOsystem(char *cmd)
 {
      char **argv, i;
      int    result;
@@ -884,9 +900,7 @@ FIOsystem(cmd)
  */
 
 FileIO
-*FIOopenCmdline(cmd, rw)
-  char *cmd;
-  char *rw;
+*FIOopenCmdline(char *cmd, char *rw)
 {
      char   **argv;
      FileIO *fio;
